# V-OS Smart Token Plugin

V-OS Token cordova plugin(com-vkey-vtap) provides a complete solution for advanced authentication and authorization on mobile phones. With the V-OS and V-OS App Protection as a security foundation, V-OS Token SDK provides a secure second-factor authentication solution on mobile in a convenient and user-friendly package. The V-OS Token SDK is available as a library for enterprises to integrate into their mobile apps to support user authentication, transaction authorization and secure communication.

V-OS Token solution architecture is premised on the need for the mobile token to remain secure even if the underlying mobile operating system has been compromised.

V-OS OTP Token allows mobile apps to be used as an alternative means to hardware OTP token that supports three modes of OTP generation, i.e. Time Based, Challenge Response, and Transaction Signing.

V-OS PKI Token is an all-in-one authentication and secure communication facilitator built over the core principles of PKI (public key infrastructure). It can validate the user identity with PKI signature. Each PKI signature can only be generated by a unique token tightly bound to the user’s device after the user approves it with a secret PIN chosen by the user themselves, or with personal biometrics kept only on the device.

## Getting Started

These instructions will get you a copy of the project up and running on your local machine for development and testing purposes.

### Prerequisites

- cordova: `> 7`
- cordova-android: `>= 6.4 < 7`
- cordova-ios: `>= 4`

Make sure that you have Cordova CLI installed on your machine. For instructions on how to get Cordova set up, please refer to Apache Cordova official documentation [here](https://cordova.apache.org/docs/en/latest/).


These are the plugins needed for Android Platform:
```
permision plugin (eg -cordova-plugin-permission)
Push Notifications plugin (eg- phonegap-plugin-push >= 2.0.0 < 2.1.2)
```

These are the plugins needed for iOS Platform:
```
Push Notifications plugin (phonegap-plugin-push)
```

### Installation

Install the plugins by executing the setup scripts provided.

Navigate to the project folder

```
cd /path/to/your/project/YourCordovaProject/
```

You can also add the individual plugins and platforms manually

##### Android
```
cordova platform add android@6.4.0
cordova plugin add cordova-plugin-dialogs
cordova plugin add cordova-plugin-permission
cordova plugin add phonegap-plugin-push@2.1.2
cordova plugin add ../com-vkey-vtap
```


##### iOS
```
cordova platform add ios 
cordova plugin add cordova-plugin-dialogs
cordova plugin add phonegap-plugin-push --variable SENDER_ID="Not Required for iOS"
cordova plugin add ../com-vkey-vtap
```

Build the project

```
cordova build
```

*For Android make sure that the Java version of project is set to 1.8 for both source and target in order to build the cordova-plugin-permissions*


Import/Open the generated project in Xcode/Android Studio

Run the test application

## Methods

Sample method calls for each V-OS Smart Token SDK API

### getInstance

```
VTapPlugin.getInstance (
		// successcb
		function (successcbmsg) {
			if(successcbmsg) {
	  			console.log("getInstance: successcb");
	  		}
	  	},
		// errorcb
		function (errorcbmsg) {
			if(errorcbmsg) {
				var msg = errorcbmsg;
	  			console.log("onAppPause errorcb : " + errorcbmsg.values);
  				console.warn("-- on vtap error");
			}
		}
	);
```


### setHostName

```
	VTapPlugin.setHostName (
		// successcb
		function (successcbmsg) {
			if(successcbmsg) {
	  			console.log("setHostName: successcb");
	  		}
	  	}, 
		// errorcb
		function (errorcbmsg) {
			if(errorcbmsg) {
				var msg = errorcbmsg;
	  			console.log("setHostName errorcb : " + errorcbmsg.values);
			},
		},
		provServer,
		vtapServer
	);
```

### setPKIHostName

```
	VTapPlugin.setPKIHostName (
		// successcb
		function (successcbmsg) {
			if(successcbmsg) {
	  			console.log("setPKIHostName: successcb");
	  		}
	  	}, 
		
		// errorcb
		function (errorcbmsg) {
			if(errorcbmsg) {
				var msg = errorcbmsg;
	  			console.log("setHostName errorcb : " + errorcbmsg.values);
			}
		}
	);
```

### setUpVTap

```
	VTapPlugin.setupVTap (		
		// successcb
		function (successcbmsg) {
			if(successcbmsg) {
	  			var msg = successcbmsg;
	  			console.log("setupVTap: successcb");
	  			console.log("-- on vtap initialize");
	  		}
		},
		// errorcb
		function (errorcbmsg) {
			if(errorcbmsg) {
				var msg = errorcbmsg;
	  			console.log("setupVTap errorcb : " + errorcbmsg.values);
  				console.warn("-- on vtap error");
			}
		}
	);
```


### getTrustedTime

```
	VTapPlugin.prototype.getTrustedTime (
		function(success) {
			console.log("getTrustedTime: successcb");
		},
		function(error) {
			//Handle error msg here
		}
	);

```

### checkDeviceCompatibility

```
	VTapPlugin.checkDeviceCompatibility (
  		function (success) {
	  		console.log("checkDeviceCompatibility: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		}
	);
```

### sendDeviceInfo

```
	VTapPlugin.sendDeviceInfo (
  		function (success) {
	  		console.log("sendDeviceInfo: successcb");
		}, 
  		function (error) {
  			//Handle error msg here
  		}
	);
```

### isProvisioningDone

```
	VTapPlugin.isProvisioningDone (
  		function (success) {
	  		console.log("isProvisioningDone: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		}
	);
```

### getAllProvisionedTokens

```
	VTapPlugin.getAllProvisionedTokens (
  		function (success) {
	  		console.log("getAllProvisionedTokens: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		}
	);
```

### getTokenType

```
	VTapPlugin.getTokenType (
  		function (success) {
	  		console.log("getTokenType: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### getDFPHash

```
	VTapPlugin.getDFPHash (
  		function (success) {
	  		console.log("getDFPHash: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		}
	);
```

### getLoadAckTokenFirmware

```
	VTapPlugin.getLoadAckTokenFirmware (  
		function (success) {
	  		console.log("getLoadAckTokenFirmware: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial,
  		apin
	);
```

### loadTokenFirmware

```
	VTapPlugin.loadTokenFirmware (
  		function (success) {
	  		console.log("loadTokenFirmware: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial,
  		apin,
  		tokenFileName
	);
```

### validateChecksum

```
	VTapPlugin.validateChecksum (
  		function (success) {
	  		console.log("validateChecksum: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial, 
  		apin
	);
```

### getPKITokenSerial

```
	VTapPlugin.getPKITokenSerial (
  		function (success) {
	  		console.log("getPKITokenSerial: successcb");
  		},
  		function (error) {
  			//Handle error msg here
  		}
	);
```

###  getTokenSerial

```
	VTapPlugin.getTokenSerial (
  		function (success) {
	  		console.log("getTokenSerial: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		}
	);
```

### getTokenFirmwareVersion

```
	VTapPlugin.getTokenFirmwareVersion (
  		function (success) {
	  		console.log("getTokenFirmwareVersion: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### removeTokenFirmware

```
	VTapPlugin.removeTokenFirmware (
  		function (success) {
	  		console.log("removeTokenFirmware: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### loadToken

```
	VTapPlugin.loadToken (
  		function (success) {
	  		console.log("loadToken: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### unloadToken

```
	VTapPlugin.unLoadToken (
  		function (success) {
	  		console.log("unLoadToken: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```


### createTokenPin

```
	VTapPlugin.createTokenPin (
  		function (success) {
	  		console.log("createTokenPin: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### isTokenPinRemembered

```
	VTapPlugin.isTokenPinRemembered (
  		function (success) {
	  		console.log("isTokenPinRemembered: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### checkTokenPin

```
	VTapPlugin.checkTokenPin ( 
  		function (success) {
	  		console.log("checkTokenPiin: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial,
  		pin,
  		rememberPin
	);
```

### checkTokenPinSeed

```
	VTapPlugin.checkTokenPinSeed (
  		function (success) {
	  		console.log("checkTokenPinSeed: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial, 
  		pin, 
  		rememberPin, 
  		seedNumber
	);
```

### changeTokenPin

```
	VTapPlugin.changeTokenPin (
  		function (success) {
	  		console.log("changeTokenPin: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial,
  		oldPin, 
  		newPin
	);
```

### isTokenRegistered

```
	VTapPlugin.isTokenRegistered (
  		function (success) {
	  		console.log("checkTokenPiinSeed: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		tokenSerial
	);
```

### setOtpLength

```
	VTapPlugin.setOtpLength (
  		function (success) {
	  		console.log("setOtpLength: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		otpLength
	);
```

### generateTOTP

```
	VTapPlugin.generateTOTP (
  		function (success) {
	  		console.log("generateTOTP: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		keyNum
	);
```

### generateCR

```
	VTapPlugin.generateCR (
  		function (success) {
	  		console.log("generateCR: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		challangeValue
	);
```

### generateTxS

```
	VTapPlugin.generateTxS (
  		function (success) {
	  		console.log("setOtpLength: successcb");
  		}, 
  		function (error) {
  			//Handle error msg here
  		},
  		account,
  		amount
	);
```

### generateCsrAndSend

```
	VTapPlugin.generateCsrAndSend (
		function(success) {
		  console.log("generateCsrAndSend: successcb");
		},
		function(error) {
		  //Handle error msg here
		},
		functionId,
		distinguishedName,
		pin,
		rememberPin
	);
```

### triggerCertPushNotification

```
	VTapPlugin.triggerCertPushNotification (
		function(success) {
		  console.log("triggerCertPushNotification: successcb");
		},
		function(error) {
		  //Handle error msg here
		},
		functionId
	);
```

### pkiCertDownload

```
	VTapPlugin.pkiCertDownload (
		function(success) {
		  console.log("pkiCertDownload: successcb");
		},
		function(error) {
		  //Handle error msg here
		},
		functionId,
		messageId,
	messageType
	);
```

### generateCsrAndSendSync

```
	VTapPlugin.generateCsrAndSendSync (
		function(success) {
		  console.log("generateCsrAndSendSync: successcb");
		},
		function(error) {
		  //Handle error msg here	          
		},
		functionId,
		distinguishedName,
		pin,
		rememberPin
	);
```

### isPKIFunctionRegistered

```
	VTapPlugin.isPKIFunctionRegistered (
		function(success) {
		  console.log("isPKIFunctionRegistered: successcb");
		},
		function(error) {
		  //Handle error msg here          
		},
		functionId
	);
```

### isPkIFunctionPinRemembered

```
    VTapPlugin.isPKIFunctionPinRemembered (
      function(success) {
          console.log("isPKIFunctionPinRemembered: successcb");
      },
      function(error) {
          //Handle error msg here
      	},
      functionId
    );
```

### pkiFunctionCheckPin

```
    VTapPlugin.pkiFunctionCheckPin (
      function(success) {
          console.log("pkiFunctionCheckPin: successcb");
      },
      function(error) {
          //Handle error msg here
      	},	          
      functionId,
      pin,
      rememberPin
    );
```

### pkiFunctionChangePin

```
    VTapPlugin.pkiFunctionChangePin (
      function(success) {
          console.log("pkiFunctionChangePin: successcb");
      },
      function(error) {
          //Handle error msg here
      	},	          
      functionId,
      oldPin,
      newPin
    );
```

### removePKIFunction

```
    VTapPlugin.removePKIFunction (
      function(success) {
          console.log("removePKIFunction: successcb");
      },
      function(error) {
          //Handle error msg here
      	},	          
      functionId
    );
```

### pushNotificationRegister

```
    VTapPlugin.pushNotificationRegister (
      function(success) {
          console.log("pushNotificationRegister: successcb");
      },
      function(error) {
          //Handle error msg here
      	},	          
      userId, 
      deviceId, 
      pushToken
    );
```

### getRegisteredUserId

```
    VTapPlugin.getRegisteredUserId (
      function(success) {
          console.log("getRegisteredUserId: successcb");
      },
      function(error) {
          //Handle error msg here
      }
    );
```

### getRegisteredDeviceId

```
    VTapPlugin.getRegisteredDeviceId (
      function(success) {
          console.log("getRegisteredDeviceId: successcb");
      },
      function(error) {
          //Handle error msg here
      }
    );
```

### pkiFunctionAuthenticate

```
    VTapPlugin.pkiFunctionAuthenticate (
      function(success) {
          console.log("pkiFunctionAuthenticate: successcb");
      },
      function(error) {
          //Handle error msg here
      	},	          
      messageId, 
      dataToBeSigned, 
      reject
    );
```

### vMessageDecrypt

```
    VTapPlugin.vMessageDecrypt = function(successcb, errorcb, encryptedMsg){
      function(success) {
          console.log("vMessageDecrypt: successcb");
      },
      function(error) {
          //Handle error msg here
      	},	          
      encryptedMsg
    );
```

### vMessageDownload

```
    VTapPlugin.vMessageDownload (
      function(success) {
          console.log("vMessageDownload: successcb");
      },
      function(error) {
          //Handle error msg here
      	},
      messageId,
      messageType,
      messageType
    );
```

### vMessageAck

```
    VTapPlugin.vMessageAck (
      function(success) {
          console.log("vMessageAck: successcb");
      },
      function(error) {
          //Handle error msg here
      	},
      messageId
    );
```

### setAdditionalData

```
    VTapPlugin.setAdditionalData (
      function(success) {
          console.log("setAdditionalData: successcb");
      },
      function(error) {
      	//Handle error msg here
      },
      type,
      value
    );
```

### getAdditionalData

```
    VTapPlugin.getAdditionalData (
      function(success) {
          console.log("getAdditionalData: successcb");
      },
      function(error) {
          //Handle error msg here
      	},
      type
    );
```

### disableAdditionalData

```
    VTapPlugin.disableAdditionalData (
      function(success) {
          console.log("disableAdditionalData: successcb");
      },
      function(error) {
          //Handle error msg here
      	},
      type
    );
```

### setMessageTypeData

```
    VTapPlugin.setMessagedataType (
      function(success) {
	      console.log("getRegisteredDeviceId: successcb");
	      if(successcb) {
	        successcb(success);
	      }
      },
      function(error) {
          //Handle error msg here
      	},
      data
    );
```

##Lifecycle Action APIs


### onAppResume

```
	VTapPlugin.onAppResume (		
		// successcb
		function (successcbmsg) {
			if(successcbmsg) {
	  			console.log("onAppResume: successcb");
	  		}
	  	},
		// errorcb
		function (errorcbmsg) {
			if(errorcbmsg) {
				var msg = errorcbmsg;
	  			console.log("onAppResume errorcb : " + errorcbmsg.values);
  				console.warn("-- on vtap error");
			}
		}
	);
```

### onAppPause

```
	VTapPlugin.onAppPause (		
		// successcb
		function (successcbmsg) {
			if(successcbmsg) {
	  			console.log("onAppPause: successcb");
	  		}
	  	},	
		// errorcb
		function (errorcbmsg) {
			if(errorcbmsg) {
				var msg = errorcbmsg;
	  			console.log("onAppPause errorcb : " + errorcbmsg.values);
  				console.warn("-- on vtap error");
			}
		}
	);
```

## Troublshooting APIs
### getTroubleshootingId
```
	VTapPlugin.getTroubleShootingId (
		function(success) {
		  console.log("getTroubleShootingId: successcb");
		},
		function(error) {
		  //Handle error msg here
		}
	);
```


### sendTroubleshootingLogs
```
	VTapPlugin.prototype.sendTroubleShootingLogs (
		function(success) {
		  console.log("getRegisteredDeviceId: successcb");
		},
		function(error) {
		 //Handle error msg here
		}
	);
```

## Response Codes

The V-OS Smart Token may return the following response codes. For response codes not within the list, contact V-Key.

* ####40200: `VTAP_SETUP_SUCCESS`
The V-OS Smart Token SDK setup succeeds. This setup includes: 

	* Successfully started V-OS and V-OS App Protection.
	* Successfully set the V-OS Smart Token Server as the trusted time server. 
	* Loaded and Initialized token firmware if the provisioning is done.

* ####40201: `VTAP_LOAD_SO_FAILED`
The .so files are packaged and delivered separately to support multiple architectures, i.e. `armeabi-v7a` and `x86`. Therefore, the `.so` files are removed from the `V-Tap.jar` and VGuard.jar files. Make sure the `.so` files are in the
	`/libs/jniLibs/` folder under the appropriate architecture. All architecture folders should contain all the `.so` files. The `.so` files are loaded by Android automatically. The system will not mix and load the `.so` files.
	
* ####40202: `VGUARD_FAILED`
	* Missing/Corrupted profile.
	* Profile decryption error: check if profile is downloaded from the right customer ID (CID) in the V-OS App Protection Server.
	* Missing/Corrupted signature.
	* Signature decryption error.
	* Invalid signature.
	* Emulator detected: if the app is running in emulator, set property `VGuardFactory.debug = true`;.

* ####40203: `VOS_FAILED`
	* Missing/Corrupted firmware. 
	* Missing/Corrupted license.
	* Signer certificate of license mismatch: check if the app is signed by the correct private keystore whose public keystore is used to generate the license.
	* Package name in license mismatch. 
	* Error in license verification.

* ####40204: `VTAP_TOKEN_LOAD_FAILED`
	* V-OS is locked due to security issues (e.g. `voscodesign.vky` file mismatch).
	* Load TA failed because `vtapta.bin` file is missing in app files directory, corrupted, or already TA Loaded.
	* Initialize TA failed because of TA secure store errors (check if DFP# changed). 

* ####40205: `VTAP_SETUP_FAILED`
V-OS Smart Token SDK setup may fail because setting trusted time server URL failed. The failure can happen when V-OS is locked due to threats detected by V- OS App Protection, or any other security issue like `voscodesign.vky` file mismatch. Make sure that the device is not locked by using the troubleshooting ID of the device in the V-OS App Protection Server, and unlock the device if it is locked.

* ####40206: `VGUARD_PERMISSION_NOT_GRANTED`
The V-OS App Protection Server needs phone, location, and storage permissions to work. Make sure that {if $os=="Android"} you setup the appropriate permissions in the `AndroidManifest.xml` file and {/if}the app user grants all three permissions during operation.

* ####40300: `VTAP_WHITE_LISTED_DEVICE`
The device compatibility check operation validates the current device against a list of whitelisted devices and returns the device as a whitelisted device. You can check your current device model number from the V-OS Smart Token Server database to know if the device is a whitelisted device.

* ####40301: `VTAP_BLACK_LISTED_DEVICE`
The device compatibility check operation returns the current device as a blacklisted device which means the host app cannot proceed to use the V-OS Smart Token SDK. You should set the host app to quit. If the host app ignores the blacklisted device, the V-OS Smart Token SDK may not behave as expected.

* ####40302: `VTAP_GREY_LISTED_DEVICE`
The device compatibility check operation returns the current device as a greylisted device which means the current device does not fall under the blacklisted device and whitelisted device. The current device is considered as a new greylisted device and host app needs to call `sendDeviceInfo` API if the app user wants to continue at his/her own risk to use the V-OS Smart Token SDK on a greylisted device.

* ####40303: `VTAP_GET_DEVICE_LIST_FAILED`
When running `checkDeviceCompatibility` API, the V-OS Smart Token SDK tries to connect to V-OS Smart Token Server to get the latest device list for comparison. The get device list web service call may fail due to reasons like:
	* V-OS Smart Token Server is down or not responding (HTTP status code 500 or any other HTTP status code other than 200).
	* Client authentication failed.

	The operation is not able to get customer ID when V-OS is not running. Make
sure that `setupVTap` is called and responded successfully.

* ####40400: `VTAP_SEND_DEVICE_INFO_SUCCESS`
The send device info operation succeed. This operation sends the device info (platform, model, and OS) along with customer ID, DFP#, greylist, and status.
The possible statuses are as follows:
	* Status = `0`: send from V-OS Smart Token SDK on the device compatibility check
	* Status = `1`: send from host app after user wants to continue at his/her own risk for greylisted device
	* Status = `2`: send from V-OS Smart Token SDK `loadTokenFirmware` method after provisioning succeed for greylisted device
	* Status = `3`: send from V-OS Smart Token SDK `loadTokenFirmware` method after provisioning failed for greylisted device
* ####40401: `VTAP_SEND_DEVICE_INFO_FAILED`
The send device info operation failed because:
	* V-OS Smart Token Server is down or not responding (HTTP status code 500 or any other HTTP status code other than 200).
	* Client authentication failed.	

	The operation is not able to get customer ID when V-OS is not running. Make
sure that `setupVTap` is called and responded successfully.
* ####40600: `VTAP_TOKEN_DOWNLOAD_SUCCESS`
The download firmware operation succeeded. The operation includes:
	* Download firmware from the V-OS Provisioning Server succeeds (HTTP Response 200).
	* Able to load the downloaded firmware file using `loadTokenFirmware` API. 
	* Send acknowledgment to the V-OS Provisioning Server succeed.
* ####40601: `VTAP_ERROR_INVALID_PROVISIONING_INFO`
The load token firmware operation failed because the input parameter
`provisioningInfo` is:
	* 	null 
	* 	empty
	* 	the size of array list is not equal to two (provisioning info array list is expected to have two items, namely token serial and APIN.)
	* 	items in the array list are null
* ####40602: `VTAP_ERROR_INVALID_TOKEN_SERIAL`
The load token firmware operation failed because the token serial of the input parameter `provisioningInfo` is:
	* 	null
	* 	empty
	* 	token serial length is not equal to 10
* ####40603: `VTAP_ERROR_INVALID_ACTIVATION_PIN`
The load token firmware operation failed because the APIN of the input
parameter `provisioningInfo` is:
	* null
	* empty
	* APIN length is not equal to 16
* ####40604: `VTAP_TOKEN_DOWNLOAD_FAILED`
The load token firmware operation failed because:
	* V-OS Provisioning Server is down. (HTTP status code 500, or no response) 
	* Client authentication failed. 
	
	The operation is not able to get customer ID when V-OS is not running. Make sure that setupVTap is called and responded successfully.
* ####40605: `VTAP_TOKEN_NOT_FOUND`
The load token firmware operation failed because:
	* Received HTTP status code 204 from the V-OS Provisioning Server.
	* The firmware is not found or already provisioned. 
	* Invalid token serial.
* ####40606: `VTAP_TOKEN_BAD_REQUEST`
The load token firmware operation request failed because:
	* Received HTTP status code 400 from the V-OS Provisioning Server. 
	* Bad request.
	* Invalid DFP#.
	* Invalid customer.
* ####40607: `VTAP_TOKEN_INVALID_DFP`
V-OS returns null DFP because V-OS is locked due to various security reasons. 
* ####40608: `VTAP_LOAD_FIRMWARE_SUCCESS`
The load firmware operation succeeded. The operation includes: 
	* Unzip the downloaded file.
	* Load TA.
	* Initialize TA.
	* Process manifest using APIN, DFP#.
	* Cleanup files: delete the manifest file, downloaded zip file.
	* Stores preference data to indicate provisioning is done successfully. 
	* Sends device info if grey listed device indicating provisioning status.
* ####40609: `VTAP_TOKEN_UNZIP_FAILED`
The token unzip operation failed because: 
	* Unzip of provisioned file failed.
	* File not found in the app files directory.
	
	Make sure that you check the ZIP file that is passed as the parameter. The downloaded token may somehow be corrupted too. You can get the host app to download another token.
* ####40610: `VTAP_TOKEN_PROCESSING_FAILED`
The token processing operation failed because of invalid APIN or DFP#, or
manifest file missing in the extracted ZIP file path.

	Make sure that you check the manifest file. The downloaded token may somehow be corrupted too. You can get the host app to download another token.
* ####40611: `VTAP_LOAD_FIRMWARE_FAILED`
The load firmware operation failed because that V-OS is not running or locked.
Make sure that `setupVTap` is called and responded successfully.

* ####40700: `VTAP_CREATE_PIN_SUCCESS`
The PIN is created in the TA secure store successfully. 
* ####40701: `VTAP_CREATE_PIN_FAILED`
The create PIN operation failed because: 
	* V-OS is not running or locked.
	* The PIN length is incorrect.
* ####40702: `VTAP_PIN_ALREADY_EXIST`
The input PIN exists. Make sure that the PIN is not created. If the PIN is created, use `changeTokenPIN` API to set a new PIN by passing the old PIN created. If you forget the old PIN and it cannot be recovered, delete the provisioned token by calling the `removeTokenFirmware` API and do provisioning again.
* ####40800: `VTAP_CHECK_PIN_SUCCESS` 
	The check PIN operation succeed. 
* ####40801: `VTAP_CHECK_PIN_FAILED`
The check PIN operation failed because: 
	* V-OS is not running or locked.
	* The PIN is incorrect.
* ####40802: `VTAP_VOS_LOCKED`
V-OS is locked due to security reasons. Make sure that the device is not locked using the troubleshooting ID of the device in the V-OS App Protection Server. If the device is locked, unlock the device. In case of `checkTokenPIN` API is called with too many wrong PIN attempts, a delay will be induced by TA, and the device is finally locked. Restart from setup V-OS Smart Token steps and use the correct PIN.
* ####40900: `VTAP_CHANGE_PIN_SUCCESS`
The PIN changed operation succeeded.
* ####40901: `VTAP_CHANGE_PIN_FAILED`
The change PIN operation failed because: 
	* The PIN length is not correct.
	* `setupVTap` call is not successful
* ####40902: `VTAP_CHANGE_INCORRECT_PIN_FAILED`
The input of `oldPin` parameter of `changePIN` API does not match the existing PIN. 
* ####41000: `VTAP_SET_OTP_LENGTH_SUCCESS`
The set OTP length operation succeed. If the `setOTPLength` is not called, the default OTP length 8.
* ####41001: `VTAP_SET_OTP_LENGTH_FAILED`
The OTP length is not set correctly. Use the `setOTPLength` API to set the OTP
length. The supported OTP length is `6`, `7`, or `8`. 
* ####41002: `VTAP_GENERATE_TOTP_FAILED`
The V-OS Smart Token SDK is not able to generate OTP, because TA is not loaded/initialized, or V-OS is not running. Make sure that setupVTap is called and responded successfully.
* ####41003: `VTAP_TOTP_UNSUPPORTED_KEY_OPTION`
The key option is not supported. The supported key option is `1` or `2`. 
* ####41004: `VTAP_GENERATE_CR_FAILED`
The V-OS Smart Token SDK is not able to generate CR because TA is not loaded/initialized, or V-OS is not running. Make sure that `setupVTap` is called and responded successfully.
* ####41005: `VTAP_CR_UNSUPPORTED_MSG_LENGTH`
The V-OS Smart Token SDK is not able to generate CR because the parameter {if
$os=="Android"}`Msg`{else if $os=="iOS"}`crMsg`{/if} of the `generateCR` API is
	* null
	* empty
	* length equals (=) zero (0) 
	* length is larger than (>) 40

	Make sure that the length of the input parameter is smaller than or equal to (<=) 40.
* ####41006: `VTAP_GENERATE_TXS_FAILED`
The V-OS Smart Token SDK is not able to generate OTP because TA is not loaded/initialized, or V-OS is not running. Make sure that setupVTap is called and responded successfully.
* ####41007: `VTAP_TXS_UNSUPPORTED_MSG_LENGTH`
The V-OS Smart Token SDK is not able to generate OCRA because the transaction
account number or amount is: 
	* null
	* empty
	* all zeros
	* invalid number
	* length equals (=) zero (0)
	* combined length of account number and amount is larger than (>) 40
	
	``Make sure that the length of the input parameter is smaller than or equal to (<=) 40.
* ####41008: `VTAP_UNSUPPORTED_OTP_LENGTH`
The V-OS Smart Token SDK is not able to generate OTP because the OTP length is
not supported. The supported OTP length is `6`, `7`, or `8`.
* ####41009: `VTAP_INCORRECT_API_CALL_SEQUENCE`
The V-OS Smart Token SDK is not able to generate OTP due to: 
	* 	`checkTokenPIN` API is not called
	* 	`createTokenPIN` process is not done (user PIN not created) 
	* 	`isTokenPINRemembered` is not set (`isRememberPin` is not set)
* ####41010: `VTAP_TIME_FAILURE`
V-OS Smart Token SDK has no secure time. Due to this reason, it is unable to generate OTP. For V-OS Smart Token SDK to obtain a trusted time, it needs to have a reference time. The reference time is obtained from the V-OS Smart Token Server. With the reference time, it is able to obtain trusted time locally without the need to contact to V-OS Smart Token Server.

	This stored reference time becomes invalid anytime due to the scenarios below, which forces the V-OS Smart Token SDK to get reference time from the server again.
	* The device is rebooted.
	* The device date/time is changed manually or automatically by the telco.
	* The device is changed to airplane mode, etc.
	* Other unforeseen reasons which will impact the device time.
	* The host app calls setupVTap API, but the reference time is not available on that device or invalid.
	
	When you receive this error without meeting one of the scenarios above, you need to obtain the trusted time again by calling the getTrustedTime API (applicable to Android only) to obtain the trusted time from the V-OS Smart Token Server.
	
	When this occurs, an asynchronous network request is made to get the reference time from the V-OS Smart Token Server. This allows the V-OS Smart Token SDK to obtain the reference time before the host app calls any V-OS OTP Token API for OTP generation. If this network response is not available due to network/server issues, another asynchronous network request is made to get ready for the next OTP generation call and response code `41010` to the host app. The host app should give some time interval (say, 500 ms between every retry if response code `41010` incurs) to allow V-OS Smart Token SDK to get the server reference time.

* ####41011: `VTAP_INVALID_INPUT`
The validation operation failed in `createTokenPIN`, `checkTokenPIN`,`changeTokenPIN`, etc.
* ####41012: `VTAP_ERROR_CONNECTION_FAILED`
Any API which tries to connect to servers and device not connected to internet returns this error.
* ####41103: `VTAP_PKI_AUTHENTICATION_FAILED`
	* 	The app is not able to do authentication signing. 
	* 	Signature verification failed at the server.
	* 	JSON object creation failed.
* ####41105: `VTAP_PKI_AUTHENTICATION_CERT_NOT_AVAILABLE`
The host app tries to do authentication signing but the certificate for AS is not
present on the device.
* ####41111: `VTAP_PKI_MESSAGE_DOWNLOAD_FAILED` 
Message download failed.
* ####41112: `VTAP_PKI_CERT_DOWNLOAD_SUCCESS`
The host app is able to download the certificate successfully.
* ####41113: `VTAP_PKI_CERT_DOWNLOAD_FAILED`
The host app is unable to download the certificate due to error.
* ####41114: `VTAP_PKI_CHECK_PIN_SUCCESS`
PIN validation succeeded.
* ####41115: `VTAP_PKI_CHECK_PIN_FAILED` 
	* V-OS is not running or locked.
	* Incorrect PIN
* ####41116: `VTAP_PKI_CHANGE_PIN_SUCCESS`
Change PIN is done successfully.
* ####41117: `VTAP_PKI_CHANGE_PIN_FAILED` 
	* 	V-OS is not running or locked.
* ####41119: `VTAP_PKI_CHANGE_INCORRECT_PIN_FAILED`
Incorrect PIN.
* ####41123: `VTAP_PKI_CERT_CHAIN_VERIFICATION_FAILED`
	* Certificate chain verification failed. 
	* Signature verification failed.
	* V-OS is locked.
* ####41125: `VTAP_VMESSAGE_DECRYPT_MSG_FAILED`
	* Decryption of message failed. 
	* V-OS is unable to decrypt. 
	* V-OS is locked.
* ####41126: `VTAP_VMESSAGE_SEND_ACK_SUCCESS`
Acknowledgment is sent to the server successfully.
* ####41127: `VTAP_VMESSAGE_SEND_ACK_FAILED` 
	* Acknowledgment to server failed.
	* Error or HTTP status code not equal to 200.
	* JSON object creation failed.
* ####41121: `VTAP_PKI_VMESSAGE_CERT_NOT_AVAILABLE`
The host app tries to decrypt the message when V-OS Messaging certificate is not present on the device.
* ####41131: `VTAP_VGUARD_OUT_OF_MEMORY`
V-OS App Protection encountered out of memory error. 
* ####41140: `VTAP_PKI_TRIGGER_SEND_CERT_SUCCESS`
The push notification request trigger succeed.
* ####41141: `VTAP_PKI_TRIGGER_SEND_CERT_FAILED`
The push notification request trigger failed.
* ####41160: `VTAP_PKI_AUTHENTICATION_REJECTION_SUCCESS` 
Authentication rejection is sent from the client successfully. 
* ####41161: `VTAP_PKI_AUTHENTICATION_REJECTION_FAILED` 
Authentication rejection from client failed.

## API Reference

Please refer to the Developer guide and API guide documentation provided by V-Key for more information on the native API

## Versioning
V-OS Smart Token SDK - Release 4.4.0.0

## Authors

* **Terence Peh**
* **La Hy Tai (Tai)**
* **Khant Zaw Ko**

## License

```
(C) Copyright 2018 V-Key Solutions.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
```

## Acknowledgments

### Third party plugin used to build the test application
* cordova-plugin-device - <https://www.npmjs.com/package/cordova-plugin-device>
* cordova-plugin-dialogs - <https://github.com/apache/cordova-plugin-dialogs>
* cordova-plugin-permission - <https://www.npmjs.com/package/cordova-plugin-permission>
* phonegap-plugin-push - <https://github.com/phonegap/phonegap-plugin-push>
