package vkey.android.cordova;

import android.Manifest;
import android.content.pm.PackageManager;
import android.util.Base64;
import android.util.Log;

import com.vkey.android.vtap.VTapInterface;
import com.vkey.android.vtap.pki.DistinguishedName;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class VTapPlugin extends CordovaPlugin {

    // VTap setup
    private static final String ACT_GET_INSTANCE                      = "act_get_instance";
    private static final String ACT_SETUP_VTAP                        = "act_setup_vtap";

    private static final String ACT_SET_HOSTNAME                      = "act_set_hostname";
    private static final String ACT_SET_PKI_HOSTNAME                  = "act_set_pki_hostname";
    private static final String ACT_GET_TRUSTED_TIME                  = "act_get_trusted_time";
    private static final String ACT_DEVICE_COMPATIBILITY              = "act_device_compatibility";
    private static final String ACT_SEND_DEVICE_INFO                  = "act_send_device_info";

    private static final String ACT_PROVISIONING_DONE                 = "act_provisioning_done";
    private static final String ACT_ALL_PROVISIONED_TOKEN             = "act_all_provisioned_token";
    private static final String ACT_TOKEN_TYPE                        = "act_token_type";
    private static final String ACT_DFP_HASH                          = "act_dfp_hash";
    private static final String ACT_GET_LOAD_ACK_TOKEN_FIRMWARE       = "act_get_load_ack_token_firmware";
    private static final String ACT_LOAD_TOKEN_FIRMWARE               = "act_load_token_firmware";
    private static final String ACT_VALIDATE_CHECKSUM                 = "act_validate_checksum_true";
    private static final String ACT_GET_PKI_TOKEN_SERIAL              = "get_pki_token_serial";
    private static final String ACT_GET_TOKEN_SERIAL                  = "act_get_token_serial";
    private static final String ACT_TOKEN_FIRMWARE_VERSION            = "act_token_firmware_version";
    private static final String ACT_REMOVE_TOKEN_FIRMWARE             = "act_remove_token";
    private static final String ACT_LOAD_TOKEN                        = "act_load_token";
    private static final String ACT_UNLOAD_TOKEN                      = "act_unload_token";
    private static final String ACT_CREATE_TOKEN_PIN                  = "act_set_token_pin";
    private static final String ACT_TOKEN_PIN_REMEMBERED              = "act_token_pin_remembered";
    private static final String ACT_CHECK_TOKEN_PIN                   = "act_check_token_pin";
    private static final String ACT_CHECK_TOKEN_PIN_SEED              = "act_check_token_pin_seed";
    private static final String ACT_CHANGE_TOKEN_PIN                  = "act_change_token_pin";
    private static final String ACT_TOKEN_REGISTERED                  = "act_token_registered";

    private static final String ACT_SET_OTP_LENGTH                    = "act_set_otp_length";
    private static final String ACT_GENERATE_TOTP                     = "act_generate_totp";
    private static final String ACT_GENERATE_CR                       = "act_generate_cr";
    private static final String ACT_GENERATE_TXS                      = "act_generate_txs";

    private static final String ACT_GENERATE_CSR_AND_SEND             = "act_generate_csr_and_send";
    private static final String ACT_TRIGGER_CERT_PUSH_NOTIFICATION    = "act_trigger_cert_push_notification";
    private static final String ACT_PKI_CERT_DOWNLOAD                 = "act_pki_cert_download";
    private static final String ACT_GENERATE_CSR_AND_SEND_SYNC        = "act_generate_csr_and_send_sync";
    private static final String ACT_PKI_FUNCTION_REGISTERED           = "act_pki_function_registered";
    private static final String ACT_PKI_FUNCTION_PIN_REMEMBERED       = "act_pki_function_pin_remembered";
    private static final String ACT_PKI_FUNCTION_CHECK_PIN            = "act_pki_function_check_pin";
    private static final String ACT_PKI_FUNCTION_CHANGE_PIN           = "act_pki_function_change_pin";
    private static final String ACT_REMOVE_PKI_FUNCTION               = "act_remove_pki_function";
    private static final String ACT_PUSH_NOTIFICATION                 = "act_push_notification";
    private static final String ACT_REGISTERED_USER_ID                = "act_registered_user_id";
    private static final String ACT_REGISTERED_DEVICE_ID              = "act_registered_device_id";
    private static final String ACT_PKI_FUNCTION_AUTHENTICATE         = "act_pki_function_authenticate";
    private static final String ACT_PKI_FUNCTION_DOCSIGN              = "act_pki_function_docsign";
    private static final String ACT_VMESSAGE_DECRYPT                  = "act_vmessage_decrypt";
    private static final String ACT_VMESSAGE_DOWNLOAD                 = "act_vmessage_download";
    private static final String ACT_VMESSAGE_ACK                      = "act_message_ack";
    private static final String ACT_SET_ADDITIONAL_DATA               = "act_set_additional_data";
    private static final String ACT_GET_ADDITIONAL_DATA               = "act_get_additional_data";
    private static final String ACT_DISABLE_ADDITIONAL_DATA           = "act_disable_additional_data";
    private static final String ACT_SET_MESSAGE_DATA_TYPE             = "act_set_message_data_type";

    private static final String ACT_VTAP_PAUSE                        = "act_vtap_pause";
    private static final String ACT_VTAP_RESUME                       = "act_vtap_resume";
    private static final String ACT_VTAP_DESTORY                      = "act_vtap_destroy";

    private static final String ACT_GET_TROUBLESHOOTING_ID            = "act_get_troubleshooting_id";
    private static final String ACT_SEND_TROUBLESHOOTING_LOGS         = "act_send_troubleshooting_logs";

    private static final int PERMISSIONS_VTAP                         = 1001;

    private String provServer;
    private String vtapServer;
    private String pkiServer;

    private VTapSetupPlugin mVtPlugin;

    private CallbackContext mCallbackContext;

    private static final String TAG = VTapPlugin.class.getSimpleName();

    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
    }

    @Override
    protected void pluginInitialize() {
        mVtPlugin = VTapSetupPlugin.getsInstance(this);
    }

    @Override
    public boolean execute(String action, String rawArgs, CallbackContext callbackContext) throws JSONException {
        mCallbackContext = callbackContext;

        if(action == null) {
            mCallbackContext.sendPluginResult(new PluginResult(PluginResult.Status.ERROR, "Action cannot be null"));
            return false;
        }
        else if(action.equals(ACT_GET_INSTANCE)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getInstance(callbackContext);
                }
            });
        }
        else if(action.equals(ACT_SETUP_VTAP)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    provServer = getArgAtIndex(args, index++);
                    vtapServer = getArgAtIndex(args, index++);
                    pkiServer = getArgAtIndex(args, index);

                    mVtPlugin.setUpVTap(callbackContext, provServer, vtapServer, pkiServer);
                }
            });
        }
        else if(action.equals(ACT_SET_HOSTNAME)){
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String provServer = getArgAtIndex(args, index++);
                    String vtapServer = getArgAtIndex(args, index);
                    mVtPlugin.setHostName(callbackContext, provServer, vtapServer);
                }
            });
        }
        else if(action.equals(ACT_SET_PKI_HOSTNAME)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String pkiServer = getArgAtIndex(args, 0);
                    mVtPlugin.setPKIHostName(callbackContext, pkiServer);
                }
            });
        }
        else if(action.equals(ACT_GET_TRUSTED_TIME)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getTrustedTime(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_DEVICE_COMPATIBILITY)){
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.checkDeviceCompatibility(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_SEND_DEVICE_INFO)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String deviceInfo = getArgAtIndex(args,index++);
                    int status = Integer.valueOf(getArgAtIndex(args, index));
                    mVtPlugin.sendDeviceInfo(callbackContext, deviceInfo, status);
                }
            });
        }
        else if (action.equals(ACT_PROVISIONING_DONE)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.isProvisioningDone(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_ALL_PROVISIONED_TOKEN)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getAllProvisionedTokens(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_TOKEN_TYPE)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String tokenSerial = getArgAtIndex(args, 0);
                    mVtPlugin.getTokenType(callbackContext, tokenSerial);
                }
            });
        }
        else if (action.equals(ACT_DFP_HASH)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getDFPHash(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_GET_LOAD_ACK_TOKEN_FIRMWARE)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String ts = getArgAtIndex(args, index++);
                    String apin = getArgAtIndex(args, index);
                    ArrayList<String> provisioningInfo = new ArrayList<String>();
                    provisioningInfo.add(ts);
                    provisioningInfo.add(apin);

                    mVtPlugin.getLoadAckTokenFirmware(callbackContext, provisioningInfo);
                }
            });
        }
        else if (action.equals(ACT_LOAD_TOKEN_FIRMWARE)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String ts = getArgAtIndex(args, index++);
                    String apin = getArgAtIndex(args, index);
                    ArrayList<String> provisioningInfo = new ArrayList<String>();
                    provisioningInfo.add(ts);
                    provisioningInfo.add(apin);
                    String fileName = getArgAtIndex(args, 2);

                    mVtPlugin.loadTokenFirmware(callbackContext, provisioningInfo, fileName);
                }
            });
        }
        else if (action.equals(ACT_VALIDATE_CHECKSUM)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String ts = getArgAtIndex(args, index);
                    String apin = getArgAtIndex(args, index);
                    ArrayList<String> provisioningInfo = new ArrayList<String>();
                    provisioningInfo.add(ts);
                    provisioningInfo.add(apin);
                    
                    mVtPlugin.validateCheckSum(callbackContext, provisioningInfo);
                }
            });
        }
        else if (action.equals(ACT_GET_PKI_TOKEN_SERIAL)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getPKITokenSerial(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_GET_TOKEN_SERIAL)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getTokenSerial(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_TOKEN_FIRMWARE_VERSION)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String ts = getArgAtIndex(args, 0);
                    mVtPlugin.getTokenFirmwareVersion(callbackContext, ts);
                }
            });
        }
        else if (action.equals(ACT_REMOVE_TOKEN_FIRMWARE)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String ts = getArgAtIndex(args, 0);
                    mVtPlugin.removeTokenFirmware(callbackContext, ts);
                }
            });
        }
        else if (action.equals(ACT_LOAD_TOKEN)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String ts = getArgAtIndex(args, 0);
                    mVtPlugin.loadToken(callbackContext, ts);
                }
            });
        }
        else if (action.equals(ACT_UNLOAD_TOKEN)) {
            cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String ts = getArgAtIndex(args, 0);
                    mVtPlugin.unloadToken(callbackContext, ts);
                }
            });
        }
        else if (action.equals(ACT_CREATE_TOKEN_PIN)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String pin = getArgAtIndex(args, index++);
                    String ts = getArgAtIndex(args, index);
                    mVtPlugin.createTokenPin(callbackContext, pin, ts);
                }
            });
        }
        else if (action.equals(ACT_TOKEN_PIN_REMEMBERED)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String ts = getArgAtIndex(args, 0);
                    mVtPlugin.isTokenPinRemembered(callbackContext, ts);
                }
            });
        }
        else if (action.equals(ACT_CHECK_TOKEN_PIN)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String pin = getArgAtIndex(args, index++);
                    boolean rememberPin = Boolean.parseBoolean(getArgAtIndex(args, index++));
                    String ts = getArgAtIndex(args, index);
                    mVtPlugin.checkTokenPin(callbackContext, pin, rememberPin, ts);
                }
            });
        }
        else if (action.equals(ACT_CHECK_TOKEN_PIN_SEED)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String pin = getArgAtIndex(args, index++);
                    boolean rememberPin = Boolean.parseBoolean(getArgAtIndex(args, index++));
                    String ts = getArgAtIndex(args, index++);
                    int seedNumber = Integer.parseInt(getArgAtIndex(args, index));
                    mVtPlugin.checkTokenPin(callbackContext, pin, rememberPin, ts, seedNumber);
                }
            });
        }
        else if (action.equals(ACT_CHANGE_TOKEN_PIN)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String oldPin = getArgAtIndex(args, index++);
                    String newPin = getArgAtIndex(args, index++);
                    String ts = getArgAtIndex(args, index);
                    mVtPlugin.changeTokenPin(callbackContext, oldPin, newPin, ts);
                }
            });
        }
        else if (action.equals(ACT_TOKEN_REGISTERED)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String ts = getArgAtIndex(args, 0);
                    mVtPlugin.isTokenRegistered(callbackContext, ts);
                }
            });
        }
        else if (action.equals(ACT_SET_OTP_LENGTH)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int length = Integer.parseInt(getArgAtIndex(args, 0));
                    mVtPlugin.setOtpLength(callbackContext, length);
                }
            });
        }
        else if (action.equals(ACT_GENERATE_TOTP)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int keyNum = Integer.parseInt(getArgAtIndex(args, 0));
                    mVtPlugin.generateTOTP(callbackContext, keyNum);
                }
            });
        }
        else if (action.equals(ACT_GENERATE_CR)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String challengeValue = getArgAtIndex(args, 0);
                    mVtPlugin.generateCR(callbackContext, challengeValue);
                }
            });
        }
        else if (action.equals(ACT_GENERATE_TXS)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String account = getArgAtIndex(args, index++);
                    String ammount = getArgAtIndex(args, index);
                    mVtPlugin.generateTxS(callbackContext, account, ammount);
                }
            });
        }
        else if (action.equals(ACT_GENERATE_CSR_AND_SEND)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    int functionId = Integer.parseInt(getArgAtIndex(args, index++));
                    String dn = getArgAtIndex(args, index++);
                    String pin = getArgAtIndex(args, index++);
                    boolean rememberPin = Boolean.parseBoolean(getArgAtIndex(args, index));

                    JSONObject jsonObject = null;
                    DistinguishedName distinguishedName = new DistinguishedName();

                    try {
                        jsonObject = new JSONObject(dn);
                        distinguishedName.setCountry(jsonObject.getString("country"));
                        distinguishedName.setStateName(jsonObject.getString("stateName"));
                        distinguishedName.setLocalityName(jsonObject.getString("localityName"));
                        distinguishedName.setOrganizationName(jsonObject.getString("organizationName"));
                        distinguishedName.setOrganizationUnit(jsonObject.getString("organizationUnit"));
                        distinguishedName.setEmailAddress(jsonObject.getString("emailAddress"));
                    } catch (JSONException e) {
                        callbackContext.error(e.getMessage());
                    }
                    mVtPlugin.generateCsrAndSend(callbackContext, functionId, distinguishedName, pin, rememberPin);
                }
            });

        }
        else if (action.equals(ACT_TRIGGER_CERT_PUSH_NOTIFICATION)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int functionId = Integer.parseInt(getArgAtIndex(args, 0));
                    mVtPlugin.triggerCertPushNotification(callbackContext, functionId);
                }
            });
        }
        else if (action.equals(ACT_PKI_CERT_DOWNLOAD)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    int functionId = Integer.parseInt(getArgAtIndex(args, index++));
                    String messageId = getArgAtIndex(args, index++);
                    String messageType = getArgAtIndex(args, index);
                    mVtPlugin.pkiCertDownload(callbackContext, functionId, messageId, messageType);
                }
            });
        }
        else if (action.equals(ACT_GENERATE_CSR_AND_SEND_SYNC)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    int functionId = Integer.parseInt(getArgAtIndex(args, index++));
                    String dn = getArgAtIndex(args, index++);
                    String pin = getArgAtIndex(args, index++);
                    boolean rememberPin = Boolean.parseBoolean(getArgAtIndex(args, index));

                    JSONObject jsonObject = null;
                    DistinguishedName distinguishedName = new DistinguishedName();

                    try {
                        jsonObject = new JSONObject(dn);
                        distinguishedName.setCountry(jsonObject.getString("country"));
                        distinguishedName.setStateName(jsonObject.getString("stateName"));
                        distinguishedName.setLocalityName(jsonObject.getString("localityName"));
                        distinguishedName.setOrganizationName(jsonObject.getString("organizationName"));
                        distinguishedName.setOrganizationUnit(jsonObject.getString("organizationUnit"));
                        distinguishedName.setEmailAddress(jsonObject.getString("emailAddress"));
                    } catch (JSONException e) {
                        callbackContext.error(e.getMessage());
                    }
                    mVtPlugin.generateCsrAndSendSync(callbackContext, functionId, distinguishedName, pin, rememberPin);
                }
            });

        }
        else if (action.equals(ACT_PKI_FUNCTION_REGISTERED)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int functionId = Integer.parseInt(getArgAtIndex(args, 0));
                    mVtPlugin.isPKIFunctionRegistered(callbackContext, functionId);
                }
            });
        }
        else if (action.equals(ACT_PKI_FUNCTION_PIN_REMEMBERED)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int functionId = Integer.parseInt(getArgAtIndex(args, 0));
                    mVtPlugin.isPKIFunctionPinRemembered(callbackContext, functionId);
                }
            });
        }
        else if (action.equals(ACT_PKI_FUNCTION_CHECK_PIN)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    int functionId = Integer.parseInt(getArgAtIndex(args, index++));
                    String pin = getArgAtIndex(args, index++);
                    boolean rememberPin = Boolean.parseBoolean(getArgAtIndex(args, index));
                    mVtPlugin.pkiFunctionCheckPin(callbackContext, functionId, pin, rememberPin);
                }
            });
        }
        else if (action.equals(ACT_PKI_FUNCTION_CHANGE_PIN)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    int functionId = Integer.parseInt(getArgAtIndex(args, index++));
                    String oldPin = getArgAtIndex(args, index++);
                    String newPin = getArgAtIndex(args, index++);
                    mVtPlugin.pkiFunctionChangePin(callbackContext, functionId, oldPin, newPin);
                }
            });
        }
        else if (action.equals(ACT_REMOVE_PKI_FUNCTION)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int functionId = Integer.parseInt(getArgAtIndex(args, 0));
                    mVtPlugin.removePKIFunction(callbackContext, functionId);
                }
            });
        }
        else if (action.equals(ACT_PUSH_NOTIFICATION)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    int index = 0;
                    String userId = getArgAtIndex(args, index++);
                    String deviceId = getArgAtIndex(args, index++);
                    String pushToken = getArgAtIndex(args, index);
                    mVtPlugin.pushNotificationRegister(callbackContext, userId, deviceId, pushToken);
                }
            });
        }
        else if (action.equals(ACT_REGISTERED_USER_ID)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getRegisteredUserId(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_REGISTERED_DEVICE_ID)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getRegisteredDeviceId(callbackContext);
                }
            });
        }
        else if (action.equals(ACT_PKI_FUNCTION_AUTHENTICATE)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    int index = 0;
                    List<String> args = getArgsList(rawArgs);
                    String messageId = getArgAtIndex(args, index++);
                    String dataToBeSigned = getArgAtIndex(args, index++);
                    boolean reject = Boolean.parseBoolean(getArgAtIndex(args, index));
                    mVtPlugin.pkiFunctionAuthenticate(callbackContext, messageId, dataToBeSigned, reject);
                }
            });
        }
        else if(action.equals(ACT_PKI_FUNCTION_DOCSIGN)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    int index = 0;
                    List<String> args = getArgsList(rawArgs);
                    String messageId = getArgAtIndex(args, index++);
                    String dataToBeSigned = getArgAtIndex(args, index++);
                    boolean reject = Boolean.parseBoolean(getArgAtIndex(args, index));
                    mVtPlugin.pkiFunctionDocSign(callbackContext, messageId, dataToBeSigned, reject);
                }
            });
        }
        else if (action.equals(ACT_VMESSAGE_DECRYPT)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String msg = getArgAtIndex(args, 0);
                    byte [] encryptedMsg = Base64.decode(msg, Base64.DEFAULT);
                    mVtPlugin.vMessageDecrypt(callbackContext, encryptedMsg);
                }
            });
        }
        else if (action.equals(ACT_VMESSAGE_DOWNLOAD)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    int index = 0;
                    List<String> args = getArgsList(rawArgs);
                    String messageId = getArgAtIndex(args, index++);
                    String messageType = getArgAtIndex(args, index++);
                    String messageFlag = getArgAtIndex(args, index);
                    mVtPlugin.vMessageDownload(callbackContext, messageId, messageType, messageFlag);
                }
            });
        }
        else if (action.equals(ACT_VMESSAGE_ACK)){
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String messageId = getArgAtIndex(args, 0);
                    mVtPlugin.vMessageAck(callbackContext, messageId);
                }
            });
        }
        else if (action.equals(ACT_SET_ADDITIONAL_DATA)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    int index = 0;
                    List<String> args = getArgsList(rawArgs);
                    String type = getArgAtIndex(args, index++);
                    String value = getArgAtIndex(args, index);
                    mVtPlugin.setAdditionalData(callbackContext, VTapInterface.AdditionDataType.valueOf(type), value);
                }
            });
        }
        else if (action.equals(ACT_GET_ADDITIONAL_DATA)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String type = getArgAtIndex(args, 0);
                    mVtPlugin.getAdditionalData(callbackContext, VTapInterface.AdditionDataType.valueOf(type));
                }
            });
        }
        else if (action.equals(ACT_DISABLE_ADDITIONAL_DATA)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String type = getArgAtIndex(args, 0);
                    mVtPlugin.disableAdditionalData(callbackContext, VTapInterface.AdditionDataType.valueOf(type));
                }
            });
        }
        else if (action.equals(ACT_SET_MESSAGE_DATA_TYPE)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    List<String> args = getArgsList(rawArgs);
                    String data = getArgAtIndex(args, 0);
                    mVtPlugin.setMessageDataType(callbackContext, data);
                }
            });
        }
        else if(action.equals(ACT_VTAP_PAUSE)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.onPause(callbackContext);
                }
            });
        }
        else if(action.equals(ACT_VTAP_RESUME)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.onResume(callbackContext);
                }
            });
        }
        else if(action.equals(ACT_VTAP_DESTORY)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.onDestroy();
                }
            });
        }
        else if(action.equals(ACT_GET_TROUBLESHOOTING_ID)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.getTroubleShootingId(callbackContext);
                }
            });
        }
        else if(action.equals(ACT_SEND_TROUBLESHOOTING_LOGS)) {
            this.cordova.getThreadPool().execute(new Runnable() {
                @Override
                public void run() {
                    mVtPlugin.sendTroubleShootingLogs(callbackContext);
                }
            });
        }
        else {
            return false;
        }
        return true;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mVtPlugin.unRegisterBroadcastReceivers();
        mVtPlugin.onDestroy();
    }

    private List<String> getArgsList(String rawArgs) {
        List<String> list = new LinkedList<String>();
        JSONArray args;
        try {
            args = new JSONArray(rawArgs);
            if(args != null && args.length()>0) {
                for (int i=0; i<args.length(); i++) {
                    list.add(args.getString(i));
                }
            }
        }
        catch (JSONException e) {
            Log.d(TAG, "-- getArgsList" + e.getMessage());
        }
        return list;
    }

    private String getArgAtIndex(List<String> args, int index) {
        String value = "";
        try {
            value = args.get(index);
            value = value.equals("null") ? null : value;
        }
        catch(IndexOutOfBoundsException ioe) {
            ioe.printStackTrace();
        }
        return value;
    }

    private byte[] hexStringToByteArray(String input) {
        byte [] output = new byte[input.length() / 2];
        for(int i = 0;  i < output.length; ++i) {
            int index = i * 2;
            output[i] = (byte) ((Character.digit(input.charAt(index), 16) << 4) +
                                Character.digit(input.charAt(index + 1), 16));
        }

        return output;
    }

    private static String bytesToHex(byte[] data) {
        if (data == null) {
            return null;
        }

        int len = data.length;
        String str = "";
        for (int i = 0; i < len; i++) {
            if ((data[i] & 0xFF) < 16)
                str = str + "0" + java.lang.Integer.toHexString(data[i] & 0xFF);
            else
                str = str + java.lang.Integer.toHexString(data[i] & 0xFF);
        }
        return str;
    }
}
