#import "VTapPlugin.h"
#import <objc/runtime.h>
#import <V-Tap/ResultCode.h>

#define REGISTRATION_TOKEN               @"Registration Token"
#define NOTIFICATION_MESSAGE_KEY         @"additionalData"
#define APS_KEY                          @"aps"
#define APP_STATE_KEY                    @"Application State"
#define TRANSACTION_ID                   @"appTransactionId"
#define NOTIFICATION_MESSAGE             @"requestMessage"
#define MESSAGE_TYPE                     @"messageType"
#define TYPE_SMP                         @"smpcleartext"

#define NOTIFICATION_ERROR               @"notification error"
#define TYPE_AUTHENTICATION              @"AUTH"
#define TYPE_DOC_SIGNING                 @"DOCSIGN"
#define TYPE_VMESSAGE                    @"SMP_MSG"
#define TYPE_CERTIFICATE_AUTH            @"ASP_CERT"
#define TYPE_CERTIFICATE_DOCSIGN         @"ASP_DOC_CERT"
#define TYPE_CERTIFICATE_SMP             @"SMP_CERT"
#define TYPE_REGISTER_PUSH_NOTIFICATION  @"REGISTER_PUSH_NOTIFICATION"
#define TYPE_CERTIFICATE_AUTH_RENEW      @"ASP_CERT_RENEW"
#define TYPE_CERTIFICATE_DOCSIGN_RENEW   @"ASPDOC_CERT_RENEW"
#define TYPE_CERTIFICATE_SMP_RENEW       @"SMP_CERT_RENEW"
#define TYPE_MESSAGE_TYPE                @"MESSAGE_TYPE"

#define DATA_TO_BE_SIGNED                @"dataToBeSigned"
#define TYPE_REMOVE_TOKEN                @"TOKEN_REPROV"

#define SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(v)  ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] != NSOrderedAscending)

@interface VTapPlugin (){
    VTapManager *vtapMgr;
    NSString* provisioningServer;
    NSString* vtapServer;
    NSString* pkiServer;
    NSMutableDictionary * notificationInfoDictionary;
    NSMutableString* docDownloadPath;
}

@property (strong, nonatomic) NSString* callbackId;
@end

@implementation VTapPlugin

- (void)pluginInitialize {
    NSLog(@"Starting VTap plugin");
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(applicationDidFinishLaunching:)
                                                 name:UIApplicationDidFinishLaunchingNotification object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(applicationDidBecomeActive:)
                                                 name:UIApplicationDidBecomeActiveNotification object:nil];
}

- (void) applicationDidFinishLaunching:(NSNotification *) notification {
    NSDictionary* launchOptions = notification.userInfo;
    if (launchOptions == nil) {
        launchOptions = [NSDictionary dictionary];
    }
}

-(void) didStartVTap:(int)statusCode{
    if (statusCode == VTAP_SETUP_SUCCESS) {
        NSLog(@"Setup VTap success");
        
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                             messageAsInt:statusCode];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:self.callbackId];
    }
    else
    {
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                             messageAsInt:statusCode];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:self.callbackId];
    }
}

- (void) applicationDidBecomeActive:(NSNotification *) notification {
    // [FBSDKAppEvents activateApp];
}

#pragma mark - Cordova commands

// /********************* Setup and Lifecycle action *********************/
//      ACT_GET_TRUSTED_TIME : "act_get_trusted_time",
-(void)act_get_instance:(CDVInvokedUrlCommand *)command{
    vtapMgr = [VTapManager sharedInstance];
    vtapMgr.delegate = self;
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                      messageAsString:@"sharedInstance"];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(void)act_setup_vtap:(CDVInvokedUrlCommand *)command {
    self.callbackId = command.callbackId;
    [self.commandDelegate runInBackground:^{
        [vtapMgr setupVTap];
    }];
}

-(void)act_vtap_resume:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        [vtapMgr onResume];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:@"OnResume"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_vtap_pause:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        [vtapMgr onPause];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:@"onPause"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_vtap_destroy:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        [vtapMgr onDestroy];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:@"onDestroy"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_hostname:(CDVInvokedUrlCommand *)command{
    NSString* provisioningServer = [[command arguments] objectAtIndex:0];
    NSString* vtapServer = [[command arguments] objectAtIndex:1];
    [self.commandDelegate runInBackground:^{
        [vtapMgr setHostName:provisioningServer vtapServer:vtapServer];
        NSString *string = [NSString stringWithFormat:@"setHostName --- provisioningServer: %@ -- vtapServer: %@",provisioningServer,vtapServer];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:string];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_pki_hostname:(CDVInvokedUrlCommand *)command{
    NSString* pkiServer = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        [vtapMgr setPKIHostName:pkiServer];
        NSString *string = [NSString stringWithFormat:@"setPKIHostName: %@", pkiServer];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:string];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_ti_server_url:(CDVInvokedUrlCommand *)command{
    NSString* tiServer = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        [vtapMgr setThreatIntelligenceServerURL:tiServer];
        NSString *string = [NSString stringWithFormat:@"setThreatIntelligenceServerURL: %@", tiServer];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:string];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_get_trusted_time:(CDVInvokedUrlCommand *)command{
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                      messageAsString:@"This API doesn't support in iOS"];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(void)act_set_allow_auto_sending_troubleshooting_log:(CDVInvokedUrlCommand *)command {
    Boolean allowAutoSendingTroubleshootingLog = [[[command arguments] objectAtIndex: 0] boolValue];
    [self.commandDelegate runInBackground:^{
        [vtapMgr setAllowAutoSendingTroubleshootingLog: allowAutoSendingTroubleshootingLog];
        NSString *string = [NSString stringWithFormat:@"setAllowAutoSendingTroubleshootingLog: %@", allowAutoSendingTroubleshootingLog ? @"true" : @"false"];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:string];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_allow_send_stacktrace_log:(CDVInvokedUrlCommand *)command {
    Boolean allowSendStacktraceLog = [[[command arguments] objectAtIndex: 0] boolValue];
    [self.commandDelegate runInBackground:^{
        [vtapMgr setAllowAutoSendingTroubleshootingLog: allowSendStacktraceLog];
        NSString *string = [NSString stringWithFormat:@"setAllowSendStacktraceLog: %@", allowSendStacktraceLog ? @"true" : @"false"];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:string];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_sdk_version:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        NSString *sdkVersion = [vtapMgr sdkVersion];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                            messageAsString:sdkVersion];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_reset_vos_trusted_storage:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        Boolean resetStatus = [vtapMgr resetVOSTrustedStorage];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                            messageAsBool:resetStatus];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

//  /********************* Device Compatibility action *********************/
-(void)act_device_compatibility:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        int deviceCompatibility = [vtapMgr checkDeviceCompatibility];
        CDVPluginResult *pluginResult;
        if (deviceCompatibility == VTAP_WHITE_LISTED_DEVICE) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:deviceCompatibility];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:deviceCompatibility];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_send_device_info:(CDVInvokedUrlCommand *)command{
    NSString* deviceInfo = [[command arguments] objectAtIndex:0];
    int status = [[[command arguments] objectAtIndex:1] intValue];
    
    [self.commandDelegate runInBackground:^{
        int respone = [vtapMgr sendDeviceInfo:deviceInfo status:status];
        CDVPluginResult *pluginResult;
        
        if (respone == VTAP_SEND_DEVICE_INFO_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:respone];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:respone];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

//  /********************** Provisioning action ****************************/
-(void)act_provisioning_done:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        BOOL isProvisioningDone = [vtapMgr isProvisioningDone];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                            messageAsBool:isProvisioningDone];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_all_provisioned_token:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        NSArray *array = [vtapMgr getAllProvisionedTokens];
        CDVPluginResult *pluginResult;
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                          messageAsArray:array];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_token_type:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        NSString* tokenSerial = [[command arguments] objectAtIndex:0];
        TokenType type = [vtapMgr getTokenType:tokenSerial];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                             messageAsInt:type];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_dfp_hash:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        NSString *dfp = [vtapMgr getDFPHash];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:dfp];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_get_load_ack_token_firmware:(CDVInvokedUrlCommand *)command {
    NSString* ts = [[command arguments] objectAtIndex:0];
    NSString* apin = [[command arguments] objectAtIndex:1];
    NSArray *array = [NSArray arrayWithObjects:ts,apin, nil];
    [self.commandDelegate runInBackground:^{
        int getTokenFirmwareStatus = [vtapMgr getLoadAckTokenFirmware:array];
        CDVPluginResult *pluginResult;
        if (getTokenFirmwareStatus == VTAP_LOAD_FIRMWARE_SUCCESS || getTokenFirmwareStatus == VTAP_TOKEN_DOWNLOAD_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:getTokenFirmwareStatus];
        } else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:getTokenFirmwareStatus];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_load_token_firmware:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    NSString* apin = [[command arguments] objectAtIndex:1];
    NSString* fileName = [[command arguments] objectAtIndex:2];
    NSString* filePathUrl = [NSString stringWithFormat:@"%@.json",fileName];
    
    [self.commandDelegate runInBackground:^{
        NSString *filePath = [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:filePathUrl];
        
        NSData *data = [[NSData alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:fileName ofType:@"json"]];
        [data writeToFile:filePath atomically:YES];
        int getTokenFirmwareStatus = [vtapMgr loadTokenFirmware:ts APIN:apin downloadFilePath:filePath];
        CDVPluginResult *pluginResult;
        if (getTokenFirmwareStatus == VTAP_LOAD_FIRMWARE_SUCCESS || getTokenFirmwareStatus == VTAP_TOKEN_DOWNLOAD_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:getTokenFirmwareStatus];
        } else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:getTokenFirmwareStatus];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_validate_checksum_true:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    NSString* apin = [[command arguments] objectAtIndex:1];
    
    [self.commandDelegate runInBackground:^{
        BOOL isValidateChecksumSucceded = [vtapMgr validateChecksum:[NSArray arrayWithObjects:ts, apin, nil]];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                            messageAsBool:isValidateChecksumSucceded];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)get_pki_token_serial:(CDVInvokedUrlCommand *)command{
    [self.commandDelegate runInBackground:^{
        NSString* pkiTokenSerial = [vtapMgr getPKITokenSerial];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:pkiTokenSerial];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_get_token_serial:(CDVInvokedUrlCommand *)command{
    [self.commandDelegate runInBackground:^{
        NSString* tokenSerial = [vtapMgr getTokenSerial];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:tokenSerial];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_token_firmware_version:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        NSString* firmwareVersion = [vtapMgr getTokenFirmwareVersion:ts];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:firmwareVersion];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_remove_token:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        int removeTokenResult = [vtapMgr removeTokenFirmware:ts];
        CDVPluginResult *pluginResult;
        if (removeTokenResult == VTAP_REMOVE_TOKEN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:removeTokenResult];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:removeTokenResult];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_load_token:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        int loadTokenResult = [vtapMgr loadToken:ts];
        CDVPluginResult *pluginResult;
        if (loadTokenResult == VTAP_LOAD_TOKEN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:loadTokenResult];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:loadTokenResult];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_unload_token:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        int unloadTokenResult = [vtapMgr unloadToken:ts];
        CDVPluginResult *pluginResult;
        if (unloadTokenResult == VTAP_UNLOAD_TOKEN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:unloadTokenResult];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:unloadTokenResult];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_token_pin:(CDVInvokedUrlCommand *)command{
    NSString* apin = [[command arguments] objectAtIndex:0];
    NSString* ts = [[command arguments] objectAtIndex:1];
    [self.commandDelegate runInBackground:^{
        int createTokenPinStatus = [vtapMgr createTokenPIN:apin withTokenSerial:ts];
        CDVPluginResult *pluginResult;
        if (createTokenPinStatus == VTAP_CREATE_PIN_SUCCESS || createTokenPinStatus == VTAP_PIN_ALREADY_EXIST) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:createTokenPinStatus];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:createTokenPinStatus];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_token_pin_remembered:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        Boolean isTokenPinRememberedStatus = [vtapMgr isTokenPINRemembered:ts];
        CDVPluginResult *pluginResult;
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                           messageAsBool:isTokenPinRememberedStatus];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_check_token_pin:(CDVInvokedUrlCommand *)command{
    NSString *pin = [[command arguments] objectAtIndex:0];
    BOOL rememberPin = [[[command arguments] objectAtIndex:1] boolValue];
    NSString *ts = [[command arguments] objectAtIndex:2];
    
    [self.commandDelegate runInBackground:^{
        int checkTokenPinStatus = [vtapMgr checkTokenPIN:pin remember:rememberPin withTokenSerial:ts];
        CDVPluginResult *pluginResult;
        
        if (checkTokenPinStatus == VTAP_CHECK_PIN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:checkTokenPinStatus];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:checkTokenPinStatus];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_check_token_pin_seed:(CDVInvokedUrlCommand *)command{
    NSString* pin = [[command arguments] objectAtIndex:0];
    BOOL rememberPin = [[[command arguments] objectAtIndex:1] boolValue];
    NSString* ts = [[command arguments] objectAtIndex:2];
    int seedNumber = [[[command arguments] objectAtIndex:3] intValue];
        
    [self.commandDelegate runInBackground:^{
        int checkTokenPinStatus = [vtapMgr checkTokenPIN:pin remember:rememberPin withTokenSerial:ts andSeed:seedNumber];
        CDVPluginResult *pluginResult;
        
        NSLog(@"check token pin - %i", checkTokenPinStatus);
        if (checkTokenPinStatus == VTAP_CHECK_PIN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:checkTokenPinStatus];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:checkTokenPinStatus];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_change_token_pin:(CDVInvokedUrlCommand *)command{
    NSString *oldPin = [[command arguments] objectAtIndex:0];
    NSString *newPin = [[command arguments] objectAtIndex:1];
    NSString *ts = [[command arguments] objectAtIndex:2];
    
    [self.commandDelegate runInBackground:^{
        int changePinStatus = [vtapMgr changeTokenPIN:oldPin withNewPIN:newPin withTokenSerial:ts];
        CDVPluginResult *pluginResult;
        if (changePinStatus == VTAP_CHANGE_PIN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                               messageAsBool:changePinStatus];
        }else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                               messageAsBool:changePinStatus];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_token_registered:(CDVInvokedUrlCommand *)command{
    NSString* ts = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        Boolean isTokenRegisteredStatus = [vtapMgr isTokenRegistered:ts];
        CDVPluginResult *pluginResult;
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                           messageAsBool:isTokenRegisteredStatus];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

//  /*************************** OTP action ********************************/
-(void)act_set_otp_length:(CDVInvokedUrlCommand *)command{
    int length = [[[command arguments] objectAtIndex:0] intValue];
    [self.commandDelegate runInBackground:^{
        int setOTPLengthStatus = [vtapMgr setOtpLength:length];
        CDVPluginResult *pluginResult;
        if (setOTPLengthStatus == VTAP_SET_OTP_LENGTH_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:setOTPLengthStatus];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:setOTPLengthStatus];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_generate_totp:(CDVInvokedUrlCommand *)command{
    int keyNum = [[[command arguments] objectAtIndex:0] intValue];
    [self.commandDelegate runInBackground:^{
        NSArray* array = [vtapMgr generateTOTP:keyNum];
        CDVPluginResult *pluginResult;
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                          messageAsArray:array];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_generate_cr:(CDVInvokedUrlCommand *)command{
    NSString* challengeValue = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        NSArray* array = [vtapMgr generateCR:challengeValue];
        CDVPluginResult *pluginResult;
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                          messageAsArray:array];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_generate_txs:(CDVInvokedUrlCommand *)command{
    NSString* account = [[command arguments] objectAtIndex:0];
    NSString* ammount = [[command arguments] objectAtIndex:1];
    [self.commandDelegate runInBackground:^{
        NSArray* array = [vtapMgr generateTxS:account amount:ammount];
        CDVPluginResult *pluginResult;
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                          messageAsArray:array];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_get_troubleshooting_id:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        dispatch_async(dispatch_get_main_queue(), ^(void) {
            NSString *troubleShootingId = [vtapMgr getTroubleshootingId];
            CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                              messageAsString:troubleShootingId];
            [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
        });
    }];
}

-(void)act_send_troubleshooting_logs:(CDVInvokedUrlCommand *)command {
    [self.commandDelegate runInBackground:^{
        int troubleShootingId = [vtapMgr sendTroubleshootingLogs];
        CDVPluginResult *pluginResult;
        if (troubleShootingId == VTAP_SEND_TROUBLESHOOTING_LOGS_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:troubleShootingId];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:troubleShootingId];
        }
 
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

// /*************************** PKI action ********************************/
-(void)act_generate_csr_and_send:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    NSDictionary* json = [[command arguments] objectAtIndex:1];
    NSString* pin = [[command arguments] objectAtIndex:2];
    BOOL rememberPin = [[[command arguments] objectAtIndex:3] boolValue];
    
    DistinguishedName* distinguishedName = [[DistinguishedName alloc] init] ;
    distinguishedName.country = [json objectForKey:@"country"];
    distinguishedName.stateName = [json objectForKey:@"stateName"];
    distinguishedName.localityName = [json objectForKey:@"localityName"];
    distinguishedName.organizationName = [json objectForKey:@"organizationName"];
    distinguishedName.organizationUnit = [json objectForKey:@"organizationUnit"];
    distinguishedName.emailAddress = [json objectForKey:@"emailAddress"];
    
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr generateCsrAndSend:functionId distinguishedName:distinguishedName pin:pin andRememberPin:rememberPin];
        CDVPluginResult *pluginResult;
        if (result == VTAP_PKI_CSR_REGISTRATION_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_trigger_cert_push_notification:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr triggerCertPushNotification:functionId];
        CDVPluginResult *pluginResult;
        if (result == VTAP_PKI_TRIGGER_SEND_CERT_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_cert_download:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    NSString* messageId = [[command arguments] objectAtIndex:1];
    NSString* messageType = [[command arguments] objectAtIndex:2];
    
    [self.commandDelegate runInBackground:^{
        int certDownloadResult = [vtapMgr pkiCertDownload:functionId messageId:messageId andMessageType:messageType];
        CDVPluginResult *pluginResult;
        if (certDownloadResult == VTAP_PKI_CERT_DOWNLOAD_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:certDownloadResult];
        } else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:certDownloadResult];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

- (void)act_pki_cert_download_retry:(CDVInvokedUrlCommand *)command{
    int functionId = [[command arguments] objectAtIndex: 0];
    
    [self.commandDelegate runInBackground:^{
        int certDownloadResult = [vtapMgr pkiCertDownloadRetry:functionId];
        CDVPluginResult *pluginResult;
        if (certDownloadResult == VTAP_PKI_CERT_DOWNLOAD_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:certDownloadResult];
        } else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:certDownloadResult];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_generate_csr_and_send_sync:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    NSDictionary* json = [[command arguments] objectAtIndex:1];
    NSString* pin = [[command arguments] objectAtIndex:2];
    BOOL rememberPin = [[[command arguments] objectAtIndex:3] boolValue];
    
    DistinguishedName* distinguishedName = [[DistinguishedName alloc] init] ;
    distinguishedName.country = [json objectForKey:@"country"];
    distinguishedName.stateName = [json objectForKey:@"stateName"];
    distinguishedName.localityName = [json objectForKey:@"localityName"];
    distinguishedName.organizationName = [json objectForKey:@"organizationName"];
    distinguishedName.organizationUnit = [json objectForKey:@"organizationUnit"];
    distinguishedName.emailAddress = [json objectForKey:@"emailAddress"];
    
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr generateCsrAndSendSync:functionId distinguishedName:distinguishedName pin:pin andRememberPin:rememberPin];
        CDVPluginResult *pluginResult;
        if (result == VTAP_PKI_CSR_REGISTRATION_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_function_registered:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    [self.commandDelegate runInBackground:^{
        BOOL result = [vtapMgr isPKIFunctionRegistered:functionId];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                           messageAsBool:result];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_function_pin_remembered:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    [self.commandDelegate runInBackground:^{
        BOOL result = [vtapMgr isPKIFunctionPinRemembered:functionId];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                           messageAsBool:result];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_function_check_pin:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    NSString* pin = [[command arguments] objectAtIndex:1];
    BOOL rememberPin = [[[command arguments] objectAtIndex:2] boolValue];
    
    [self.commandDelegate runInBackground:^{
        int checkFunctionPINResult = [vtapMgr pkiFunctionCheckPin:functionId withPin:pin andRememberPin:rememberPin];
        CDVPluginResult *pluginResult;
        if (checkFunctionPINResult == VTAP_PKI_CHECK_PIN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:checkFunctionPINResult];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:checkFunctionPINResult];
        }
        
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_function_change_pin:(CDVInvokedUrlCommand *)command {
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    NSString* oldPin = [[command arguments] objectAtIndex:1];
    NSString* newPin = [[command arguments] objectAtIndex:2];

    [self.commandDelegate runInBackground:^{
        int changeFunctionPINResult = [vtapMgr pkiFunctionChangePIN:functionId oldPin:oldPin withNewPIN:newPin];
        CDVPluginResult *pluginResult;
        if (changeFunctionPINResult == VTAP_PKI_CHANGE_PIN_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:changeFunctionPINResult];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:changeFunctionPINResult];
        }
        
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_remove_pki_function:(CDVInvokedUrlCommand *)command{
    int functionId = [[[command arguments] objectAtIndex:0] intValue];
    [self.commandDelegate runInBackground:^{
        [vtapMgr removePKIFunction:functionId];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                            messageAsBool:YES];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_push_notification_register:(CDVInvokedUrlCommand *)command{
    NSString* userId = [[command arguments] objectAtIndex:0];
    NSString* deviceId = [[command arguments] objectAtIndex:1];
    NSString* pushToken = [[command arguments] objectAtIndex:2];
    
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr pushNotificationRegister:userId deviceId:deviceId andToken:pushToken];
        CDVPluginResult *pluginResult;
        if (result == VTAP_PUSH_NOTIFICATION_REGISTRATION_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        }else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_push_notification_regsiter_with_type:(CDVInvokedUrlCommand *)command{
    NSString* userId = [[command arguments] objectAtIndex:0];
    NSString* deviceId = [[command arguments] objectAtIndex:1];
    NSString* pushToken = [[command arguments] objectAtIndex:2];
    int pnsType = [[[command arguments] objectAtIndex:3] intValue];
    
    [self.commandDelegate runInBackground:^{
        int result = VTAP_PUSH_NOTIFICATION_REGISTRATION_FAILED;
        if (pnsType == 0) {
            result = [vtapMgr pushNotificationRegister:userId deviceId:deviceId andToken:pushToken pnsType:APNS];
        } else if (pnsType == 1) {
            result = [vtapMgr pushNotificationRegister:userId deviceId:deviceId andToken:pushToken pnsType:FCM];
        } else if (pnsType == 2) {
            result = [vtapMgr pushNotificationRegister:userId deviceId:deviceId andToken:pushToken pnsType:HMS];
        } else {
            result = VTAP_PUSH_NOTIFICATION_REGISTRATION_FAILED;
        }
        CDVPluginResult *pluginResult;
        if (result == VTAP_PUSH_NOTIFICATION_REGISTRATION_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        }else{
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_registered_user_id:(CDVInvokedUrlCommand *)command{
    [self.commandDelegate runInBackground:^{
        NSString* userID = [vtapMgr getRegisteredUserId];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:userID];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_registered_device_id:(CDVInvokedUrlCommand *)command{
    [self.commandDelegate runInBackground:^{
        NSString* deviceId = [vtapMgr getRegisteredDeviceId];
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:deviceId];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_function_authenticate:(CDVInvokedUrlCommand *)command{
    NSString* messageId = [[command arguments] objectAtIndex:0];
    NSString* dataToBeSigned = [[command arguments] objectAtIndex:1];
    BOOL reject = [(NSString *)[[command arguments] objectAtIndex:2] boolValue];
    
    [self.commandDelegate runInBackground:^{
        int authResult = [vtapMgr pkiFunctionAuthenticate:messageId dataToBeSigned:dataToBeSigned andReject:reject];
        CDVPluginResult *pluginResult;
        if (authResult == VTAP_PKI_AUTHENTICATION_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:authResult];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:authResult];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_pki_function_doc_sign:(CDVInvokedUrlCommand *)command {
    NSString *messageId = [[command arguments] objectAtIndex: 0];
    NSString *dataToBeSigned = [[command arguments] objectAtIndex: 1];
    BOOL reject = [[[command arguments] objectAtIndex: 2] boolValue];
    
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr pkiFunctionDocSign:messageId dataToBeSigned:dataToBeSigned andReject:reject];
        CDVPluginResult *pluginResult;
        if (result == VTAP_PKI_DOC_SIGNING_SUCCESS) {
            pluginResult =  [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                 messageAsInt:result];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        
        [self.commandDelegate sendPluginResult: pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_vmessage_decrypt:(CDVInvokedUrlCommand *)command{
    NSString *notifyStr = [[command arguments] objectAtIndex:0];
    NSData *notifyData = [[NSData alloc] initWithBase64EncodedString:notifyStr options:0];
    
    [self.commandDelegate runInBackground:^{
        NSString *notifyMsg = [vtapMgr vMessageDecrypt:notifyData];
        
        CDVPluginResult *pluginResult;
        
        // CHECKING ERROR CALLBACK FIRST IN THIS CASE -- EXCEPTION
        if ([notifyMsg intValue] == VTAP_VMESSAGE_DECRYPT_MSG_FAILED ||
            [notifyMsg intValue] == VTAP_PKI_VMESSAGE_CERT_NOT_AVAILABLE ||
            [notifyMsg intValue] == VTAP_INVALID_INPUT ||
            [notifyMsg intValue] == VTAP_INVALID_API_SEQUENCE ||
            notifyMsg == nil) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                             messageAsString:notifyMsg];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                             messageAsString:notifyMsg];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_vmessage_download:(CDVInvokedUrlCommand *)command{
    NSString* messageId = [[command arguments] objectAtIndex:0];
    NSString* messageType = [[command arguments] objectAtIndex:1];
    NSString* messageFlag =  [[command arguments] objectAtIndex:2];
    
    [self.commandDelegate runInBackground:^{
        NSArray *downloadResultArray = [vtapMgr vMessageDownload:messageId messageType:messageType andMessageFlag:messageFlag];
        
        NSException *excep;
        CDVPluginResult *pluginResult;
        
        if (excep||downloadResultArray.count < 1) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                             messageAsString:@""];
        } else {
            NSString * msgDownloadPath =[downloadResultArray objectAtIndex:0];
            // CHECKING ERROR CALLBACK FIRST IN THIS CASE -- EXCEPTION
            if ([msgDownloadPath intValue] == VTAP_PKI_MESSAGE_DOWNLOAD_FAILED ||
                [msgDownloadPath intValue] == VTAP_INVALID_INPUT ||
                [msgDownloadPath intValue] == VTAP_ERROR_CONNECTION_FAILED) {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                  messageAsArray:downloadResultArray];
            } else {
                pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                  messageAsArray:downloadResultArray];
            }
        }
        
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_message_ack:(CDVInvokedUrlCommand *)command{
    NSString* messageId = [[command arguments] objectAtIndex:0];
    
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr vMessageAck:messageId];
        CDVPluginResult *pluginResult;
        if (result == VTAP_VMESSAGE_SEND_ACK_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_additional_data:(CDVInvokedUrlCommand *)command{
    NSString *type = [[command arguments] objectAtIndex:0];
    NSString *value = [[command arguments] objectAtIndex:1];
    
    [self.commandDelegate runInBackground:^{
        int setStatus  = [vtapMgr setAdditionalData:type additionalDataType:value];
        CDVPluginResult *pluginResult;
        if (setStatus == VTAP_CREATE_ADDITIONAL_DATA_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:setStatus];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:setStatus];
        }
        
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_get_additional_data:(CDVInvokedUrlCommand *)command{
    NSString* type = [[command arguments] objectAtIndex:0];
    
    [self.commandDelegate runInBackground:^{
        NSString* result = [vtapMgr getAdditionalData:type];
        
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                          messageAsString:result];
        
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_disable_additional_data:(CDVInvokedUrlCommand *)command{
    NSString* type = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        int result = [vtapMgr disableAdditionalData:type];
        CDVPluginResult *pluginResult;
        
        if (result == VTAP_DISABLE_ADDITIONAL_DATA) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:result];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:result];
        }
        
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_set_message_data_type:(CDVInvokedUrlCommand *)command{
    NSString* data = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        int setMessageTypeStatus = [vtapMgr setMessageTypeData:data];
        CDVPluginResult *pluginResult;
        if (setMessageTypeStatus == VTAP_CREATE_ADDITIONAL_DATA_SUCCESS) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                messageAsInt:setMessageTypeStatus];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsInt:setMessageTypeStatus];
        }

        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}

-(void)act_get_cert:(CDVInvokedUrlCommand *)command{
    int functionId = [[command arguments] objectAtIndex:0];
    [self.commandDelegate runInBackground:^{
        CDVPluginResult *pluginResult;
        NSData *certData = [[NSData alloc] init];
        if (functionId == 0) {
            certData = [vtapMgr getCert:CERT_TYPE_ASP];
        } else if (functionId == 1){
            certData = [vtapMgr getCert:CERT_TYPE_SMP];
        } else {
            certData = nil;
        }

        if (certData != nil) {
            NSString *certString = [[NSString alloc] initWithData:certData encoding:NSUTF8StringEncoding];
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                              messageAsString:certString];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                messageAsString:@"CertData is nil"];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    }];
}


-(BOOL)isMissingParam:(int)num command:(CDVInvokedUrlCommand *)command{
    if (command.arguments.count < num){
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                          messageAsString:@"MISSING PARAMS"];
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
        return YES;
    };
    return NO;
}

@end






