/*
 * (C) Copyright 2018 V-Key Solutions.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 *     	Terence Peh
 */

 var argscheck = require('cordova/argscheck'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec');

	VTapPlugin = function() {
	};
		
	/**
	 * Constants
	 */
	var CONST = {
		PLG_NAME : "VTapPlugin",

		/********************* Setup and Lifecycle action *********************/
		ACT_GET_INSTANCE : "act_get_instance",
		ACT_SETUP_VTAP : "act_setup_vtap",
    	ACT_VTAP_PAUSE : "act_vtap_pause",
    	ACT_VTAP_RESUME : "act_vtap_resume",
		ACT_VTAP_DESTROY : "act_vtap_destroy",
    	ACT_SET_HOSTNAME : "act_set_hostname",     
		ACT_SET_PKI_HOSTNAME : "act_set_pki_hostname",
		ACT_SET_TI_SERVER_URL : "act_set_ti_server_url",
		ACT_GET_TRUSTED_TIME : "act_get_trusted_time",
		ACT_SDK_VERSION : "act_sdk_version",
		ACT_SET_ALLOW_AUTO_SENDING_TROUBLESHOOTING_LOG : "act_set_allow_auto_sending_troubleshooting_log",
		ACT_SET_ALLOW_SEND_STACKTRACE_LOG : "act_set_allow_send_stacktrace_log",

        ACT_SETUP_THREAT_DELEGATE : "act_setup_threat_delegate",
        CB_SCAN_COMPLETE : "cb_scan_complete",
		
		/********************* Device Compatibility action *********************/
		ACT_DEVICE_COMPATIBILITY : "act_device_compatibility",
		ACT_SEND_DEVICE_INFO : "act_send_device_info",  

		/********************** Provisioning action ****************************/
		ACT_PROVISIONING_DONE : "act_provisioning_done",                  
		ACT_ALL_PROVISIONED_TOKEN : "act_all_provisioned_token",           
		ACT_TOKEN_TYPE : "act_token_type",                                 
		ACT_DFP_HASH : "act_dfp_hash",                                  
		ACT_GET_LOAD_ACK_TOKEN_FIRMWARE : "act_get_load_ack_token_firmware",
		ACT_LOAD_TOKEN_FIRMWARE : "act_load_token_firmware",            
		ACT_VALIDATE_CHECKSUM : "act_validate_checksum_true",            
		ACT_GET_PKI_TOKEN_SERIAL : "get_pki_token_serial",                  
		ACT_GET_TOKEN_SERIAL : "act_get_token_serial",                      
		ACT_TOKEN_FIRMWARE_VERSION : "act_token_firmware_version",          
		ACT_REMOVE_TOKEN_FIRMWARE : "act_remove_token",          
		ACT_LOAD_TOKEN : "act_load_token",                     
		ACT_UNLOAD_TOKEN : "act_unload_token",                              
		ACT_CREATE_TOKEN_PIN : "act_set_token_pin",                         
		ACT_TOKEN_PIN_REMEMBERED : "act_token_pin_remembered",              
		ACT_CHECK_TOKEN_PIN : "act_check_token_pin",
		ACT_CHECK_TOKEN_PIN_SEED : "act_check_token_pin_seed",                        
		ACT_CHANGE_TOKEN_PIN : "act_change_token_pin",                      
		ACT_TOKEN_REGISTERED : "act_token_registered",

		/*************************** OTP action ********************************/
		ACT_SET_OTP_LENGTH : "act_set_otp_length",
		ACT_GENERATE_TOTP : "act_generate_totp",  
		ACT_GENERATE_CR : "act_generate_cr",     
		ACT_GENERATE_TXS : "act_generate_txs",

		/*************************** PKI action ********************************/
		ACT_GENERATE_CSR_AND_SEND : "act_generate_csr_and_send",             
		ACT_TRIGGER_CERT_PUSH_NOTIFICATION : "act_trigger_cert_push_notification",
		ACT_PKI_CERT_DOWNLOAD : "act_pki_cert_download",
		ACT_PKI_CERT_DOWNLOAD_RETRY : "act_pki_cert_download_retry",  
		ACT_GENERATE_CSR_AND_SEND_SYNC : "act_generate_csr_and_send_sync",      
		ACT_PKI_FUNCTION_REGISTERED : "act_pki_function_registered",            
		ACT_PKI_FUNCTION_PIN_REMEMBERED : "act_pki_function_pin_remembered",    
		ACT_PKI_FUNCTION_CHECK_PIN : "act_pki_function_check_pin",
		ACT_PKI_FUNCTION_CHANGE_PIN : "act_pki_function_change_pin",            
		ACT_REMOVE_PKI_FUNCTION : "act_remove_pki_function",                     
		ACT_PUSH_NOTIFICATION_REGISTER : "act_push_notification_register",              
		ACT_PUSH_NOTIFICATION_REGISTER_WITH_TYPE : "act_push_notification_regsiter_with_type",                                    
		ACT_REGISTERED_USER_ID : "act_registered_user_id",                      
		ACT_REGISTERED_DEVICE_ID : "act_registered_device_id",                   
		ACT_PKI_FUNCTION_AUTHENTICATE : "act_pki_function_authenticate",
		ACT_PKI_FUNCTION_DOCSIGN : "act_pki_function_docsign",
		ACT_VMESSAGE_DECRYPT : "act_vmessage_decrypt",                         
		ACT_VMESSAGE_DOWNLOAD : "act_vmessage_download",                        
		ACT_VMESSAGE_ACK : "act_message_ack",                                  
		ACT_SET_ADDITIONAL_DATA : "act_set_additional_data",                     
		ACT_GET_ADDITIONAL_DATA : "act_get_additional_data",                     
		ACT_DISABLE_ADDITIONAL_DATA : "act_disable_additional_data",             
		ACT_SET_MESSAGE_DATA_TYPE : "act_set_message_data_type",
		ACT_GET_CERT : "act_get_cert",

		/************************* Troubleshooting APIs *************************/
		ACT_GET_TROUBLESHOOTING_ID : "act_get_troubleshooting_id", 
		ACT_SEND_TROUBLESHOOTING_LOGS : "act_send_troubleshooting_logs",
		ACT_RESET_VOS_TRUSTED_STORAGE : "act_reset_vos_trusted_storage"
	};

	VTapPlugin.prototype.getInstance = function (onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("getInstance: successcb");
					if(onSuccess) {
						onSuccess();
					}
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("onAppPause errorcb : " + errorcbmsg.values);
					console.warn("-- on vtap error");
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_GET_INSTANCE,
			[]
		);
	};


	/**
	 * Starts VTap service
	 * @param {Function} onVTapSuccess
	 * @param {Function} onVTapFailed
	 */
	VTapPlugin.prototype.setupVTap = function (onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					var msg = successcbmsg;
					console.log("setupVTap: successcb");
					console.log("-- on vtap initialize");
					if(onSuccess) {
						onSuccess(msg);
					}
						/* else if(msg && msg.action == CONST.CB_SCAN_COMPLETE) {
							console.log("-- on scan complete");
							if(onScanComplete) {
								onScanComplete(msg.values);
							}
						}
						else if(msg && msg.action == CONST.CB_VTAP_FAILED) {
							console.log("-- on vtap failed");
							if(onVTapFailed) {
								onVTapFailed();
							}
						}
						else if(msg && msg.action == CONST.CB_VTAP_ERROR) {
							// should not come here..
							console.warn("-- on vtap error");
						} */
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("setupVTap errorcb : " + errorcbmsg.values);
					console.warn("-- on vtap error");
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME, 
			CONST.ACT_SETUP_VTAP,
			[]
		);
	};

	VTapPlugin.prototype.setupThreatScanDelegate = function (onScanComplete) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					var msg = successcbmsg;
					console.log("setupThreatScanDelegate: successcb");
					if(msg && msg.action == CONST.CB_SCAN_COMPLETE) {
						console.log("-- on scan complete");
						if(onScanComplete) {
							onScanComplete(msg.values);
						}
					}
				}
			},

			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("setupThreatScanDelegate errorcb : " + errorcbmsg.values);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_SETUP_THREAT_DELEGATE,
			[]
		);
	};

	VTapPlugin.prototype.onAppPause = function (onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("onAppPause: successcb");
					if(onSuccess) {
						onSuccess();
					}
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("onAppPause errorcb : " + errorcbmsg.values);
					console.warn("-- on vtap error");
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_VTAP_PAUSE,
			[]
		);
	};
	
	VTapPlugin.prototype.onAppResume = function (onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("onAppResume: successcb");
					if(onSuccess) {
						onSuccess();
					}
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("onAppResume errorcb : " + errorcbmsg.values);
					console.warn("-- on vtap error");
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_VTAP_RESUME,
			[]
		);	
	};
	
	VTapPlugin.prototype.onAppDestroy = function (onSuccess, onError) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("onAppDestroy: successcb");
						if(onSuccess) {
							onSuccess();
						}
					}
				}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("onAppDestroy errorcb : " + errorcbmsg.values);
					console.warn("-- on vtap error");
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_VTAP_DESTROY,
			[]
		);	
	};

	VTapPlugin.prototype.setHostName = function (onSuccess, onError, provServer, vtapServer) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("setHostName: successcb");
					if(onSuccess) {
						onSuccess();
					}
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("setHostName errorcb : " + errorcbmsg.values);
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_SET_HOSTNAME,
			[provServer, vtapServer]
		);	
	};


	VTapPlugin.prototype.setPKIHostName = function (onSuccess, onError, pkiServer) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("setPKIHostName: successcb");
					if(onSuccess) {
						onSuccess();
					}
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("setHostName errorcb : " + errorcbmsg.values);
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_SET_PKI_HOSTNAME,
			[pkiServer]
		);	
	};

	VTapPlugin.prototype.setThreatIntelligenceServerURL = function (onSuccess, onError, tiServer) {
		cordova.exec(
			// successcb
			function (successcbmsg) {
				if(successcbmsg) {
					console.log("setTIServerURL: successcb");
					if(onSuccess) {
						onSuccess();
					}
				}
			}, 
			
			// errorcb
			function (errorcbmsg) {
				if(errorcbmsg) {
					var msg = errorcbmsg;
					console.log("setTIServerURL errorcb : " + errorcbmsg.values);
					if(onError) {
						onError(msg);
					}
				}
			}, 
			CONST.PLG_NAME,
			CONST.ACT_SET_TI_SERVER_URL,
			[tiServer]
		);	
	};

	VTapPlugin.prototype.getTrustedTime = function(successcb, errorcb) {
		exec(
			function(success) {
				console.log("getTrustedTime: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_GET_TRUSTED_TIME,
			[]);
	};
	
	VTapPlugin.prototype.setAllowAutoSendingTroubleshootingLog = function(successcb, errorcb, allow) {
		exec(
			function(success) {
				console.log("setAllowAutoSendingTroubleshootingLog: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_SET_ALLOW_AUTO_SENDING_TROUBLESHOOTING_LOG,
			[allow]);
	};
	
	VTapPlugin.prototype.setAllowSendStacktraceLog = function(successcb, errorcb, allow) {
		exec(
			function(success) {
				console.log("setAllowSendStacktraceLog: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_SET_ALLOW_SEND_STACKTRACE_LOG,
			[allow]);
	};
	
	VTapPlugin.prototype.sdkVersion = function(successcb, errorcb) {
		exec(
			function(success) {
				console.log("sdkVersion: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_SDK_VERSION,
			[]);
	};


	VTapPlugin.prototype.checkDeviceCompatibility = function(successcb, errorcb) {
		exec(
			function (success) {
				console.log("checkDeviceCompatibility: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_DEVICE_COMPATIBILITY,
			[]);
	};


	VTapPlugin.prototype.sendDeviceInfo = function(successcb, errorcb, deviceInfo, status) {
		exec(
			function (success) {
				console.log("sendDeviceInfo: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_SEND_DEVICE_INFO,
			[deviceInfo, status]);
	};


	VTapPlugin.prototype.isProvisioningDone = function(successcb, errorcb) {
		exec(
			function (success) {
				console.log("isProvisioningDone: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_PROVISIONING_DONE,
			[]);
	};

		
		/**
		 * check if provisioning is done
	 * @param {Object} status
		 */
		VTapPlugin.prototype.isProvisioningDone = function(successcb, errorcb) {
			exec(
				function (success) {
					console.log("isProvisioningDone: successcb");
					if(successcb) {
						successcb(success);
					}
				}, 
				function (error) {
					if(errorcb) {
						errorcb(error);
					}
				},
				CONST.PLG_NAME, 
				CONST.ACT_PROVISIONING_DONE,
				[]);
	};


	VTapPlugin.prototype.getAllProvisionedTokens = function(successcb, errorcb) {
		exec(
			function (success) {
				console.log("getAllProvisionedTokens: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_ALL_PROVISIONED_TOKEN,
			[]);
	}

	VTapPlugin.prototype.getTokenType = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
				console.log("getTokenType: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_TOKEN_TYPE,
			[tokenSerial]);
	}

	VTapPlugin.prototype.getDFPHash = function(successcb, errorcb) {
		exec(
			function (success) {
				console.log("getDFPHash: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_DFP_HASH,
			[]);
	};

	VTapPlugin.prototype.getLoadAckTokenFirmware = function(successcb, errorcb, tokenSerial, apin) {
		exec(
			function (success) {
				console.log("getLoadAckTokenFirmware: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_GET_LOAD_ACK_TOKEN_FIRMWARE,
			[tokenSerial, apin]);
	};

	VTapPlugin.prototype.loadTokenFirmware = function(successcb, errorcb, tokenSerial, apin, tokenFileName) {
		exec(
			function (success) {
				console.log("loadTokenFirmware: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_LOAD_TOKEN_FIRMWARE,
			[tokenSerial, apin, tokenFileName]);
	};


	VTapPlugin.prototype.validateChecksum = function(successcb, errorcb, tokenSerial, apin) {
		exec(
			function (success) {
				console.log("validateChecksum: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_VALIDATE_CHECKSUM,
			[tokenSerial, apin]);
	};


	VTapPlugin.prototype.getPKITokenSerial = function(successcb, errorcb) {
		exec(
			function (success) {
				console.log("getPKITokenSerial: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_GET_PKI_TOKEN_SERIAL,
			[]);
	};


	VTapPlugin.prototype.getTokenSerial = function(successcb, errorcb) {
		exec(
			function (success) {
				console.log("getTokenSerial: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_GET_TOKEN_SERIAL,
			[]);
	};


	VTapPlugin.prototype.getTokenFirmwareVersion = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
				console.log("getTokenFirmwareVersion: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_TOKEN_FIRMWARE_VERSION,
			[tokenSerial]);
	};

	VTapPlugin.prototype.removeTokenFirmware = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
			console.log("removeTokenFirmware: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_REMOVE_TOKEN_FIRMWARE,
			[tokenSerial]);
	};


	VTapPlugin.prototype.loadToken = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
			console.log("loadToken: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_LOAD_TOKEN,
			[tokenSerial]);
	};


	VTapPlugin.prototype.unLoadToken = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
			console.log("unLoadToken: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_UNLOAD_TOKEN,
			[tokenSerial]);
	};


	VTapPlugin.prototype.createTokenPin = function(successcb, errorcb, pin, tokenSerial) {
		exec(
			function (success) {
			console.log("createTokenPin: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_CREATE_TOKEN_PIN,
			[pin, tokenSerial]);
	};


	VTapPlugin.prototype.isTokenPinRemembered = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
			console.log("isTokenPinRemembered: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_TOKEN_PIN_REMEMBERED,
			[tokenSerial]);
	};


	VTapPlugin.prototype.checkTokenPin = function(successcb, errorcb, pin, rememberPin, tokenSerial) {
		exec(
			function (success) {
			console.log("checkTokenPin: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_CHECK_TOKEN_PIN,
			[pin, rememberPin, tokenSerial]);
	};


	VTapPlugin.prototype.checkTokenPinSeed = function(successcb, errorcb, pin, rememberPin, tokenSerial, seedNumber) {
		exec(
			function (success) {
			console.log("checkTokenPinSeed: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_CHECK_TOKEN_PIN_SEED,
			[pin, rememberPin, tokenSerial, seedNumber]);
	};


	VTapPlugin.prototype.changeTokenPin = function(successcb, errorcb, oldPin, newPin, tokenSerial) {
		exec(
			function (success) {
			console.log("changeTokenPin: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_CHANGE_TOKEN_PIN,
			[oldPin, newPin, tokenSerial]);
	};


	VTapPlugin.prototype.isTokenRegistered = function(successcb, errorcb, tokenSerial) {
		exec(
			function (success) {
			console.log("checkTokenPiinSeed: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_TOKEN_REGISTERED,
			[tokenSerial]);
	};


	VTapPlugin.prototype.setOtpLength = function(successcb, errorcb, otpLength) {
		exec(
			function (success) {
			console.log("setOtpLength: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_SET_OTP_LENGTH,
			[otpLength]);
	};


	VTapPlugin.prototype.generateTOTP = function(successcb, errorcb, keyNum) {
		exec(
			function (success) {
			console.log("generateTOTP: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_GENERATE_TOTP,
			[keyNum]);
	};


	VTapPlugin.prototype.generateCR = function(successcb, errorcb, challangeValue) {
		exec(
			function (success) {
			console.log("generateCR: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_GENERATE_CR,
			[challangeValue]);
	};


	VTapPlugin.prototype.generateTxS = function(successcb, errorcb, account, amount) {
		exec(
			function (success) {
			console.log("setOtpLength: successcb");
				if(successcb) {
					successcb(success);
				}
			}, 
			function (error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME, 
			CONST.ACT_GENERATE_TXS,
			[account, amount]);
	};


	VTapPlugin.prototype.generateCsrAndSend = function(successcb, errorcb, functionId, distinguishedName, pin, rememberPin){
        exec (
        	function(success) {
			console.log("generateCsrAndSend: successcb");
				if(successcb) {
					successcb(success);
				}
        	},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_GENERATE_CSR_AND_SEND,
			[functionId, distinguishedName, pin, rememberPin]);
    };


    VTapPlugin.prototype.triggerCertPushNotification = function(successcb, errorcb, functionId){
        exec (
			function(success) {
			console.log("triggerCertPushNotification: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_TRIGGER_CERT_PUSH_NOTIFICATION,
			[functionId]);
    };


    VTapPlugin.prototype.pkiCertDownload = function(successcb, errorcb, functionId, messageId, messageType){
        exec (
			function(success) {
			console.log("pkiCertDownload: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_CERT_DOWNLOAD,
			[functionId, messageId, messageType]);
    };
	
	VTapPlugin.prototype.pkiCertDownloadRetry = function(successcb, errorcb, functionId){
		exec (
			function(success) {
			console.log("pkiCertDownloadRetry: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_CERT_DOWNLOAD_RETRY,
			[functionId]);
	};

    VTapPlugin.prototype.generateCsrAndSendSync = function(successcb, errorcb, functionId, distinguishedName, pin, rememberPin){
        exec (
			function(success) {
			console.log("generateCsrAndSendSync: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_GENERATE_CSR_AND_SEND_SYNC,
			[functionId, distinguishedName, pin, rememberPin]);
    };


    VTapPlugin.prototype.isPKIFunctionRegistered = function(successcb, errorcb, functionId){
        exec (
			function(success) {
			console.log("isPKIFunctionRegistered: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_FUNCTION_REGISTERED,
			[functionId]);
    };


    VTapPlugin.prototype.isPKIFunctionPinRemembered = function(successcb, errorcb, functionId){
        exec (
			function(success) {
			console.log("isPKIFunctionPinRemembered: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_FUNCTION_PIN_REMEMBERED,
			[functionId]);
    };


    VTapPlugin.prototype.pkiFunctionCheckPin = function(successcb, errorcb, functionId, pin, rememberPin){
        exec (
			function(success) {
			console.log("pkiFunctionCheckPin: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_FUNCTION_CHECK_PIN,
			[functionId, pin, rememberPin]);
	};
	

	VTapPlugin.prototype.pkiFunctionChangePin = function(successcb, errorcb, functionId, oldPin , newPin){
        exec (
			function(success) {
			console.log("pkiFunctionChangePin: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_FUNCTION_CHANGE_PIN,
			[functionId, oldPin, newPin]);
    };


    VTapPlugin.prototype.removePKIFunction = function(successcb, errorcb, functionId){
        exec (
			function(success) {
			console.log("removePKIFunction: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_REMOVE_PKI_FUNCTION,
			[functionId]);
    };


    VTapPlugin.prototype.pushNotificationRegister = function(successcb, errorcb, userId, deviceId, pushToken){
        exec (
			function(success) {
			console.log("pushNotificationRegister: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PUSH_NOTIFICATION_REGISTER,
			[userId, deviceId, pushToken]);
    };
	
	
	VTapPlugin.prototype.pushNotificationRegisterWithType = function(successcb, errorcb, userId, deviceId, pushToken, pnsType){
		exec (
			function(success) {
			console.log("pushNotificationRegisterWithType: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PUSH_NOTIFICATION_REGISTER_WITH_TYPE,
			[userId, deviceId, pushToken, pnsType]);
	};
	

    VTapPlugin.prototype.getRegisteredUserId = function(successcb, errorcb){
        exec (
			function(success) {
			console.log("getRegisteredUserId: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_REGISTERED_USER_ID,
			[]);
    };


    VTapPlugin.prototype.getRegisteredDeviceId = function(successcb, errorcb){
		exec (
			function(success) {
			console.log("getRegisteredDeviceId: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_REGISTERED_DEVICE_ID,
			[]);
    };


    VTapPlugin.prototype.pkiFunctionAuthenticate = function(successcb, errorcb, messageId, dataToBeSigned, reject){
        exec (
			function(success) {
			console.log("pkiFunctionAuthenticate: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_FUNCTION_AUTHENTICATE,
			[messageId, dataToBeSigned, reject]);
    };


    VTapPlugin.prototype.pkiFunctionDocSign = function(successcb, errorcb, messageId, dataToBeSigned, reject){
        exec (
			function(success) {
			console.log("pkiFunctionDocSign: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_PKI_FUNCTION_DOCSIGN,
			[messageId, dataToBeSigned, reject]);
    };


    VTapPlugin.prototype.vMessageDecrypt = function(successcb, errorcb, encryptedMsg){
        exec (
			function(success) {
			console.log("vMessageDecrypt: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_VMESSAGE_DECRYPT,
			[encryptedMsg]);
    };


    VTapPlugin.prototype.vMessageDownload = function(successcb, errorcb, messageId, messageType, messageFlag){
        exec (
			function(success) {
			console.log("vMessageDownload: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_VMESSAGE_DOWNLOAD,
			[messageId, messageType, messageFlag]);
    };


    VTapPlugin.prototype.vMessageAck = function(successcb, errorcb, messageId){
        exec (
			function(success) {
			console.log("vMessageAck: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_VMESSAGE_ACK,
			[messageId]);
    };


    VTapPlugin.prototype.setAdditionalData = function(successcb, errorcb, type, value){
        exec (
			function(success) {
			console.log("setAdditionalData: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_SET_ADDITIONAL_DATA,
			[type, value]);
    };


    VTapPlugin.prototype.getAdditionalData = function(successcb, errorcb, type){
        exec (
			function(success) {
			console.log("getAdditionalData: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_GET_ADDITIONAL_DATA,
			[type]);
    };


    VTapPlugin.prototype.disableAdditionalData = function(successcb, errorcb, type){
        exec (
			function(success) {
				console.log("disableAdditionalData: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_DISABLE_ADDITIONAL_DATA,
			[type]);
    };


    VTapPlugin.prototype.setMessagedataType = function(successcb, errorcb, data){
        exec (
			function(success) {
				console.log("setMessagedataType: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_REGISTERED_USER_ID,
			[data]);
	};

	VTapPlugin.prototype.getCert = function(successcb, errorcb, certType){
		exec (
			function(success) {
				console.log("getCert: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_GET_CERT,
			[certType]);
	};
	

	VTapPlugin.prototype.getTroubleShootingId = function(successcb, errorcb){
        exec (
			function(success) {
				console.log("getTroubleShootingId: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
			    	errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_GET_TROUBLESHOOTING_ID,
			[]);
	};
	

	VTapPlugin.prototype.sendTroubleShootingLogs = function(successcb, errorcb){
        exec (
			function(success) {
				console.log("getRegisteredDeviceId: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_SEND_TROUBLESHOOTING_LOGS,
			[]);
    };
	
	VTapPlugin.prototype.resetVOSTrustedStorage = function(successcb, errorcb){
		exec (
			function(success) {
				console.log("resetVOSTrustedStorage: successcb");
				if(successcb) {
					successcb(success);
				}
			},
			function(error) {
				if(errorcb) {
					errorcb(error);
				}
			},
			CONST.PLG_NAME,
			CONST.ACT_RESET_VOS_TRUSTED_STORAGE,
			[]);
	};
	
module.exports = new VTapPlugin();
